#include "audioout48.h"

static void SrcInitialize(void);
static void Src16kSpsTo48kSps(const int16s Sample, int16s *SrcSamples);

AudioOut48::AudioOut48(QObject *parent)
 :   QIODevice(parent)
{
    produceRawOutputFile = false;
    uncompressedFile48 = NULL;
}

AudioOut48::~AudioOut48()
{
    if (produceRawOutputFile)
    {
        if (uncompressedFile48)
        {
            uncompressedFile48->close();
            delete uncompressedFile48;
        }
    }
}

void AudioOut48::start()
{
    open(QIODevice::ReadOnly);
}

void AudioOut48::stop()
{
    close();
}

void AudioOut48::resetGlobals()
{
    SrcInitialize();
    if (produceRawOutputFile)
    {
        if (uncompressedFile48)
        {
            uncompressedFile48->close();
            delete uncompressedFile48;
        }
        uncompressedFile48 = new QFile();
        uncompressedFile48->setFileName(QString("%1_48.dat").arg(QDateTime::currentDateTime().toString("MMM-dd_hh-mm-ss")));
        uncompressedFile48->open(QIODevice::WriteOnly);
        uncompressedOut48.setDevice(uncompressedFile48);
    }
}

qint64 AudioOut48::readData(char *data, qint64 len)
{
    // Read data and remove it from the buffer
    qint64 total = 0;
    mutex.lock();
    int bytesAvailable = m_buffer.count();
    mutex.unlock();
    if (bytesAvailable <= 0)
    {
        memset(data, 0x00, len);
        return len;
    }
    //
    // Read the maximum request, or maximum available, then
    // remove the bytes from the buffer
    if (len < bytesAvailable)
    {
        bytesAvailable = len;
    }
    if (bytesAvailable > 0)
    {
        mutex.lock();
        memcpy(data, m_buffer.constData(), bytesAvailable);
        m_buffer.remove(0, bytesAvailable);
        mutex.unlock();
        total += bytesAvailable;
    }
    return total;
}

qint64 AudioOut48::writeData(const char *data, qint64 len)
{
    // Broken? Problems in type casting here?
    qint64 total = 0;
    char samples[6];
    int sampleIndex = 0;
    while (len - total > 0)
    {
        // Convert the 16k sample to three samples in 48k
        Src16kSpsTo48kSps((int16s)((int8u)data[total] | ((int16s)data[total+1] << 8)), (int16s*)samples);
        mutex.lock();
        m_buffer.append(samples[sampleIndex]);
        m_buffer.append(samples[sampleIndex+1]);
        m_buffer.append(samples[sampleIndex+2]);
        m_buffer.append(samples[sampleIndex+3]);
        m_buffer.append(samples[sampleIndex+4]);
        m_buffer.append(samples[sampleIndex+5]);
        mutex.unlock();
        if (produceRawOutputFile)
        {
            uncompressedOut48 << (int8u)samples[sampleIndex];
            uncompressedOut48 << (int8u)samples[sampleIndex+1];
            uncompressedOut48 << (int8u)samples[sampleIndex+2];
            uncompressedOut48 << (int8u)samples[sampleIndex+3];
            uncompressedOut48 << (int8u)samples[sampleIndex+4];
            uncompressedOut48 << (int8u)samples[sampleIndex+5];
        }
        total+=2; // Processed 2 bytes (16 bit sample)
    }
    return total;
}

void AudioOut48::writeSample(int16s sample)
{
    int8u samples[6];
    // Convert the 16k sample to three samples in 48k
    Src16kSpsTo48kSps(sample, (int16s*)samples);
    mutex.lock();
    m_buffer.append((int8u)samples[0]);
    m_buffer.append((int8u)samples[1]);
    m_buffer.append((int8u)samples[2]);
    m_buffer.append((int8u)samples[3]);
    m_buffer.append((int8u)samples[4]);
    m_buffer.append((int8u)samples[5]);
    mutex.unlock();
    if (produceRawOutputFile)
    {
        uncompressedOut48 << (int8u)samples[0];
        uncompressedOut48 << (int8u)samples[1];
        uncompressedOut48 << (int8u)samples[2];
        uncompressedOut48 << (int8u)samples[3];
        uncompressedOut48 << (int8u)samples[4];
        uncompressedOut48 << (int8u)samples[5];
    }
}

qint64 AudioOut48::bytesAvailable() const
{
    //mutex.lock();
    qint64 bytesAvailable = m_buffer.size();// + QIODevice::bytesAvailable();
    //mutex.unlock();
    return bytesAvailable;
}

// 16k to 48k Conversion Constants
#define SRC_METHOD 1 // insert 0's and LPF
//#define SRC_METHOD 2 // linear interpolate and LPF

//NOTE: Src below has been optimized to only support SRC_RATIO=3!!!
#define SRC_RATIO (48000/16000)  // must be an integer!!!

// LPF_4POLE_Chebyshev coefs (f-3dB=7.2kHz from fc=0.15 on fs=48kSps)
static const int16s A0A4 = 97;     // note A0 and A4 are always the same
static const int16s A1A3 = 387;    // note A1 and A3 are always the same
static const int16s A2 = 580;
static const int16s B1 = 16704;
static const int16s B2 = -16490;
static const int16s B3 = 8108;
static const int16s B4 = -1677;
#define LPF_SHIFT 13

// Used for 16k to 48k conversion
#if (SRC_METHOD == 2)
static int16s SrcLastSample;
#endif
// hold LPF 4-pole z^-1, z^-2, z^-3, and z^-4
static int32s Xnm1;
static int32s Xnm2;
static int32s Xnm3;
static int32s Xnm4;
static int32s Ynm1;
static int32s Ynm2;
static int32s Ynm3;
static int32s Ynm4;

static void Src16kSpsTo48kSps(const int16s Sample, int16s *SrcSamples)
{
int8u i;
int32s x, y;

#if (SRC_METHOD == 1)
  // Note: optimized SRC_RATIO=3 solution
  SrcSamples[0] = SrcSamples[1] = 0; // insert two zeros (48kSps/16kSps-1)
  SrcSamples[2] = Sample; // copy sample
#endif

#if (SRC_METHOD == 2)
int32s Delta;

  Delta = (int32s)Sample - (int32s)SrcLastSample; // compute difference

  // Note: optimized SRC_RATIO=3 solution
  SrcSamples[0] = (int16s)(    Delta / 3 + (int32s)SrcLastSample);
  SrcSamples[1] = (int16s)(2 * Delta / 3 + (int32s)SrcLastSample);
  SrcSamples[2] = Sample;

  SrcLastSample = Sample;
#endif

  for (i=0;i<SRC_RATIO;i++) {
    x = (int32s)SrcSamples[i]; // promote from int16s to int32s
#if (SRC_METHOD == 1)
    if (i==(SRC_RATIO-1))
      x *= SRC_RATIO;		 // correct amplitude for zero insertion
#endif

    // run data through LPF (4-pole)
    y = (A0A4*x + A1A3*Xnm1 + A2*Xnm2 + A1A3*Xnm3 + A0A4*Xnm4
            + B1*Ynm1 + B2*Ynm2 + B3*Ynm3 + B4*Ynm4) >> LPF_SHIFT;
    Ynm4 = Ynm3;             // update LPF z^-1, z^-2, z^-3, and z^-4
    Ynm3 = Ynm2;
    Ynm2 = Ynm1;
    Ynm1 = y;
    Xnm4 = Xnm3;
    Xnm3 = Xnm2;
    Xnm2 = Xnm1;
    Xnm1 = x;

    // clamp sample to 16-bit value
    if (y > 32767) y = 32767;
    else if (y < -32768) y = -32768;

    // save back to buffer
    SrcSamples[i] = (int16s)y;
  }
}

//-----------------------------------------------------------------------------
static void SrcInitialize(void)
{
#if (SRC_METHOD == 2)
  SrcLastSample = 0;
#endif
  Xnm1 = Xnm2 = Xnm3 = Xnm4 = Ynm1 = Ynm2 = Ynm3 = Ynm4 = 0; // reset LPF
}
