#include "dialog.h"
#include "ui_dialog.h"

#ifdef __linux__
#include "quicksetdialog.h"
#endif //__linux__

Dialog::Dialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Dialog)
{
    ui->setupUi(this);

    QPalette backgroundPal(palette());
    backgroundPal.setColor(QPalette::Background, Qt::white);
    this->setAutoFillBackground(true);
    this->setPalette(backgroundPal);

    // Default audio types, these can be changed when reading the profile
    // for the remote, below
    customIniAudioSettings = false;
    audioDecodeType = MSADPCM;
    audioCompressedBufferSize = 0;
    offset = 0;
    gain = 0;

    // Get all the profiles to be supported in the app
    gatherProfiles();
    resetTest();

    // Setup our audio handler
    audioHandler = new RF4CEAudioHandler(this, HARDWARE_CODEC_STRING);

    // Do this after profiles have been gathered, in case custom settings
    // are in the INI file to be sure they get included in the options and
    // after the audio handler is created so if an option changes the class
    // is available
    setupAudioCodecOptions();

    // If we have custom audio codec settings make sure those get
    // set, and configured to the default
    if (customIniAudioSettings == true)
    {
        audioHandler->setCustomCodecSettings(CUSTOM_CODEC_STRING,
                                             audioDecodeType,
                                             audioCompressedBufferSize,
                                             offset,
                                             gain);
    }

    // Setup our protocol handler
    protocolHandler = new RF4CEProtocolHandler(this, audioHandler);

    // Setup our serial handler with the two protocol and
    // audio handlers we created
    serialHandler = new SerialHandler(this, protocolHandler);

    // Setup our list box with all the profiles, and select one
    currentProfile = -1;
    if (remotes.count() > 0)
    {
        currentProfile = 0;
        for (int i = 0; i < remotes.count(); i++)
        {
            ui->profileComboBox->addItem(remotes[i]->profileName);
        }
        if (ui->profileComboBox->count() >= 0)
        {
            ui->profileComboBox->setCurrentIndex(currentProfile);
        }
        else
        {
            currentProfile = -1;
        }
    }

    // Enable the QuickSet DB Lookup if we are in Linux
    ui->audioDataLabel->setVisible(false);
    ui->clearAudioPushButton->setVisible(false);
    ui->playSelectedAudioPushButton->setVisible(false);
    ui->realtimeAudioCheckBox->setVisible(false);
    ui->codecTypeComboBox->setVisible(false);
    ui->waveFileListWidget->setVisible(false);
    ui->line_audio->setVisible(false);
    ui->quickSetButton->setVisible(false);
    ui->line_quickset->setVisible(false);
    if (ui->profileComboBox->currentText() == "Universal_Electronics")
    {
        ui->audioDataLabel->setVisible(true);
        ui->clearAudioPushButton->setVisible(true);
        ui->playSelectedAudioPushButton->setVisible(true);
        ui->realtimeAudioCheckBox->setVisible(true);
        ui->codecTypeComboBox->setVisible(true);
        ui->waveFileListWidget->setVisible(true);
        ui->line_audio->setVisible(true);
#ifdef __linux__
        ui->quickSetButton->setVisible(true);
        ui->line_quickset->setVisible(true);
#endif //__linux__
    }

    // Update the COM ports
    refreshPorts();

    // Force a repaint to update the remote
    repaint();
}

void Dialog::setupAudioCodecOptions()
{
    ui->codecTypeComboBox->addItem(HARDWARE_CODEC_STRING);
    ui->codecTypeComboBox->addItem(SOFTWARE_CODEC_STRING);
    if (this->customIniAudioSettings)
    {
        ui->codecTypeComboBox->addItem(CUSTOM_CODEC_STRING);
    }
}

void Dialog::gatherProfiles()
{
    // Look in the profiles directory for any remote profiles we may have:
    // ex: ./profiles/<profile name>/<profile images/ini file>
    QDir currentDirectory = QCoreApplication::applicationDirPath();
    if (!currentDirectory.cd("profiles"))
        return;

    // Loop through the sub-dirs in profiles to gather all the supported
    // profiles
    QStringList profileList = currentDirectory.entryList(QDir::AllDirs | QDir::NoDotAndDotDot);
    for (int i = 0; i < profileList.count(); i++)
    {
        // Enter the sub-dir to grab all the INI/BMP data
        if (currentDirectory.cd(profileList[i]))
        {
            // Check for ini file and BMP files
            if (QFile::exists(currentDirectory.absoluteFilePath(REMOTE_MAIN_IMAGE_FILE)) &&
                QFile::exists(currentDirectory.absoluteFilePath(REMOTE_PRESSED_IMAGE_FILE)) &&
                QFile::exists(currentDirectory.absoluteFilePath(REMOTE_RELEASED_IMAGE_FILE)) &&
                QFile::exists(currentDirectory.absoluteFilePath(REMOTE_INI_FILE)))
            {
                // Create a new profile and fill it out
                RemoteProfile* rp = new RemoteProfile(this);
                rp->keyTestTime = 200;
                rp->totalKeyCount = 0;

                // Set the profile name to the directory name
                rp->profileName = profileList[i];

                // Store all our stock images
                rp->remoteImage = QImage(currentDirectory.absoluteFilePath(REMOTE_MAIN_IMAGE_FILE));
                rp->remoteImagePressed = QImage(currentDirectory.absoluteFilePath(REMOTE_RELEASED_IMAGE_FILE));
                rp->remoteImageReleased = QImage(currentDirectory.absoluteFilePath(REMOTE_MAIN_IMAGE_FILE));

                rp->remotePixmap = QPixmap::fromImage(rp->remoteImage);
                rp->remotePixmapPressed = QPixmap::fromImage(rp->remoteImagePressed);
                rp->remotePixmapReleased = QPixmap::fromImage(rp->remoteImageReleased);

                // Try and read the INI file and add it in to the list if it is successful
                if (readIniFile(rp, currentDirectory.absoluteFilePath(REMOTE_INI_FILE)))
                {
                    // Add the new profile to our main list
                    remotes.append(rp);
                }
            }

            // Go back up a level for the next profile
            currentDirectory.cdUp();
        }
    }
}

bool Dialog::readIniFile(RemoteProfile* rp, QString iniFilePath)
{
    QFile iniFile(iniFilePath);
    if (!iniFile.open(QIODevice::ReadWrite))
        return false;

    QTextStream in(&iniFile);
    QString line;
    while (!in.atEnd())
    {
        line = in.readLine();

        // Get the total key count
        if (line.contains("SumKey="))
        {
            rp->totalKeyCount = line.mid(line.indexOf('=') + 1).toInt();
        }

        // Get the button press test time
        if (line.contains("TestTime="))
        {
            // Assumes only 4 digit test time
            rp->keyTestTime = line.mid(line.indexOf('=') + 1, 4).trimmed().toInt() * TEST_TIME_MULTIPLIER;
        }

        // Get the audio settings
        if (line.contains("AudioDecodeType="))
        {
            customIniAudioSettings = true;
            if (line.mid(line.indexOf('=') + 1).contains("MSADPCM"))
            {
                audioDecodeType = MSADPCM;
            }
            else if (line.mid(line.indexOf('=') + 1).contains("IMADPCM"))
            {
                audioDecodeType = IMADPCM;
            }
        }
        if (line.contains("AudioCompressedBufferSize="))
        {
            customIniAudioSettings = true;
            audioCompressedBufferSize = line.mid(line.indexOf('=') + 1).toInt();
        }
        if (line.contains("Offset="))
        {
            customIniAudioSettings = true;
            offset = line.mid(line.indexOf('=') + 1).toInt();
        }
        if (line.contains("Gain="))
        {
            customIniAudioSettings = true;
            gain = line.mid(line.indexOf('=') + 1).toInt();
        }

        // Get each key
        if (line.contains("[key"))
        {
            KeyProfile* kp = new KeyProfile(this);
            // Hardcoded to 6 lines
            for (int i = 0; i < 6; i++)
            {
                line = in.readLine();
                if (line.contains("Left"))
                    kp->left = line.mid(line.indexOf('=') + 1).toInt();
                if (line.contains("Top"))
                    kp->top = line.mid(line.indexOf('=') + 1).toInt();
                if (line.contains("Right"))
                    kp->right = line.mid(line.indexOf('=') + 1).toInt();
                if (line.contains("Bottom"))
                    kp->bottom = line.mid(line.indexOf('=') + 1).toInt();
                if (line.contains("name"))
                    kp->name = line.mid(line.indexOf('=') + 1);
                if (line.contains("code"))
                    kp->code = line.mid(line.indexOf('=') + 1).toInt();
            }
            rp->keys.append(kp);
        }

        // Once we've hit our key count (more than 0) break out
        if ((rp->keys.count() > 0) &&
            (rp->keys.count() == rp->totalKeyCount))
            break;
    }
    Q_ASSERT(rp->totalKeyCount == rp->keys.count());
    iniFile.close();
    return true;
}

Dialog::~Dialog()
{
    delete protocolHandler;
    delete ui;
}

void Dialog::closeEvent(QCloseEvent *)
{
    on_comPortComboBox_currentIndexChanged(0);
}

void Dialog::paintEvent(QPaintEvent *)
{
    QPainter painter(this);

    if (currentProfile >= 0)
    {
        // Always draw the current stock image - note that all drawing/scaling requires the remote, button
        // up, and button down images to all be the same size and overlay each other perfectly
        float originalPixmapHeight = remotes[currentProfile]->remotePixmap.height();

        QPixmap remotePixmapScaled = remotes[currentProfile]->remotePixmap.scaled(QSize(ui->remoteViewWidget->rect().width(), ui->remoteViewWidget->rect().height()),
                                           Qt::KeepAspectRatio,
                                           Qt::SmoothTransformation);

        painter.drawPixmap(ui->remoteViewWidget->rect().top(),
                           ui->remoteViewWidget->rect().left(),
                           remotePixmapScaled);

        // Select overlays for key presses depedning on our flags
        QPixmap remoteActionPixmapScaled;
        if (pressingKey)
        {
            remoteActionPixmapScaled = remotes[currentProfile]->remotePixmapPressed.scaled(QSize(ui->remoteViewWidget->rect().width(), ui->remoteViewWidget->rect().height()),
                                               Qt::KeepAspectRatio,
                                               Qt::SmoothTransformation);
        }
        else if (releasingKey)
        {
            remoteActionPixmapScaled = remotes[currentProfile]->remotePixmapReleased.scaled(QSize(ui->remoteViewWidget->rect().width(), ui->remoteViewWidget->rect().height()),
                                               Qt::KeepAspectRatio,
                                               Qt::SmoothTransformation);
        }

        // If any flag was set draw our selected overlay
        if (pressingKey || releasingKey)
        {
            float scaleFactor = (float)remoteActionPixmapScaled.height() / originalPixmapHeight;

            int left = remotes[currentProfile]->keys[currentKeyIndex]->left * scaleFactor;
            int top = remotes[currentProfile]->keys[currentKeyIndex]->top * scaleFactor;
            int width = (remotes[currentProfile]->keys[currentKeyIndex]->right - remotes[currentProfile]->keys[currentKeyIndex]->left) * scaleFactor;
            int height = (remotes[currentProfile]->keys[currentKeyIndex]->bottom - remotes[currentProfile]->keys[currentKeyIndex]->top) * scaleFactor;

            painter.drawPixmap(ui->remoteViewWidget->rect().top() + left,
                               ui->remoteViewWidget->rect().left() + top,
                               remoteActionPixmapScaled.copy(QRect(left, top, width, height)));
        }
    }
}

void Dialog::on_profileComboBox_currentIndexChanged(int index)
{
    // Reste our current index, reset and repaint
    currentProfile = index;
    if (ui->profileComboBox->currentText() == "Universal_Electronics")
    {
        ui->audioDataLabel->setVisible(true);
        ui->clearAudioPushButton->setVisible(true);
        ui->playSelectedAudioPushButton->setVisible(true);
        ui->realtimeAudioCheckBox->setVisible(true);
        ui->codecTypeComboBox->setVisible(true);
        ui->waveFileListWidget->setVisible(true);
        ui->line_audio->setVisible(true);
#ifdef __linux__
        ui->quickSetButton->setVisible(true);
        ui->line_quickset->setVisible(true);
#endif //__linux__
    }
    else
    {
        ui->audioDataLabel->setVisible(false);
        ui->clearAudioPushButton->setVisible(false);
        ui->playSelectedAudioPushButton->setVisible(false);
        ui->realtimeAudioCheckBox->setVisible(false);
        ui->codecTypeComboBox->setVisible(false);
        ui->waveFileListWidget->setVisible(false);
        ui->line_audio->setVisible(false);
        ui->quickSetButton->setVisible(false);
        ui->line_quickset->setVisible(false);
    }

    resetTest();
    repaint();
}

void Dialog::on_simulateButton_clicked()
{
    if (currentProfile >= 0)
    {
        // Always simulate a non-atomic press
        keyCodePressed(currentKeyCode, 0x03);

        // Increment our code for next time
        currentKeyCode++;

        // See if we are done with the test
        if (currentKeyCode >= remotes[currentProfile]->keys.count())
        {
            if(testResult == true)
            {
                QMessageBox(QMessageBox::Information, APP_TITLE, "Test Finished PASS", QMessageBox::Ok).exec();
                resetTest();
            }
            else
            {
                QMessageBox(QMessageBox::Information, APP_TITLE, "Test Finished FAIL", QMessageBox::Ok).exec();
                resetTest();
            }
        }
    }
}

void Dialog::keyCodePressed(int keyCode, unsigned char keyFlags)
{
    // Note, this function has been put together for a demo -
    // it should be updated to take in the action keys as arguments
    // and handle state changes for the remote keyboard.
    // The keycodes -1/-2 signal the audio up down - this doesn't map
    // directly to the other key actions so it's being spoofed.

    if (keyCode == -1)
    {
        for (int i = 0; i < remotes[currentProfile]->keys.count(); i++)
        {
            if (11 ==  remotes[currentProfile]->keys[i]->code)
            {
                currentKeyIndex = i;
            }
        }
        pressingKey = true;
        //ui->keyCodeLineEdit->setText(QString::number(-1));
        //ui->keyNameLineEdit->setText("AUDIO");
        repaint();
        return;
    }
    else if (keyCode == -2)
    {
        // This is special, we need to cancel the keypress and also the audio
        // stream
        pressingKey = false;
        releasingKey = true;
        audioHandler->setStreamDone();
        repaint();
        return;
    }

    // Find the code in our test data
    for (int i = 0; i < remotes[currentProfile]->keys.count(); i++)
    {
        if (keyCode ==  (int)remotes[currentProfile]->keys[i]->code)
        {
            currentKeyIndex = i;
        }
    }

    // Set our timer if the keypress is atomic, if so set the timer, otherwise toggle
    if (keyFlags == 0x03)
    {
        // Atomic
        pressingKey = true;
        QTimer::singleShot(remotes[currentProfile]->keyTestTime, this, SLOT(on_timeout()));
    }
    else if (keyFlags == 0x01)
    {
        // Non-atomic press
        pressingKey = true;
    }
    else if (keyFlags == 0x00)
    {
        // Non-atomic release
        // Just call the time out like the timer has expired
        pressingKey = false;
        releasingKey = true;
    }

    //ui->keyCodeLineEdit->setText(QString::number(remotes[currentProfile]->keys[currentKeyIndex]->code));
    //ui->keyNameLineEdit->setText(remotes[currentProfile]->keys[currentKeyIndex]->name);
    repaint();
}

void Dialog::postTrace(QString traceString)
{
    // Controller Heartbeat packet
    if (traceString.contains("Type: 0x1e"))
    {
        controllerHeartbeat = !controllerHeartbeat;
        if (controllerHeartbeat)
        {
            ui->controllerHeartbeatLabel->setText("Controller Heartbeat: ***");
        }
        else
        {
            ui->controllerHeartbeatLabel->setText("Controller Heartbeat:");
        }
        if (!ui->showHeartbeatCheckBox->isChecked())
        {
            return;
        }
    }

    // Target Heartbeat packet
    if (traceString.contains("Type: 0x0"))
    {
        targetHeartbeat = !targetHeartbeat;
        if (targetHeartbeat)
        {
            ui->targetHeartbeatLabel->setText("Target Heartbeat: ***");
        }
        else
        {
            ui->targetHeartbeatLabel->setText("Target Heartbeat:");
        }
        if (!ui->showHeartbeatCheckBox->isChecked())
        {
            return;
        }
    }

    ui->traceDataTextEdit->append(QString("%1%2").arg(QDateTime::currentDateTime().toString("hh:mm:ss:zzz | "), traceString));
}

void Dialog::on_saveTraceDataPushButton_clicked()
{
    // Write trace data out to file
    QString dateTimeString = QDateTime::currentDateTime().toString("MMM-dd_hh-mm-ss");
    QFile logFile(QString("%1-trace_log.txt").arg(dateTimeString));
    logFile.open(QIODevice::WriteOnly);
    QTextStream textOut(&logFile);
    textOut << ui->traceDataTextEdit->toPlainText();
    logFile.close();
    QMessageBox(QMessageBox::Question, APP_TITLE, QString("Wrote trace log to %1").arg(logFile.fileName()), QMessageBox::Ok).exec();
}

void Dialog::addWaveFile(QString waveFileName)
{
    ui->waveFileListWidget->addItem(waveFileName);
}

void Dialog::on_waveFileListWidget_doubleClicked(const QModelIndex &index)
{
    int itemIndex = index.row();
    if (itemIndex >= 0)
    {
        playAudioFile(ui->waveFileListWidget->item(itemIndex)->text());
    }
}

void Dialog::on_realtimeAudioCheckBox_clicked()
{
    if (ui->realtimeAudioCheckBox->checkState() == Qt::Checked)
    {
        audioHandler->setRealTimeOutput(true);
    }
    else
    {
        audioHandler->setRealTimeOutput(false);
    }
}

void Dialog::on_playSelectedAudioPushButton_clicked()
{
    int itemIndex = ui->waveFileListWidget->currentIndex().row();
    if (itemIndex >= 0)
    {
        playAudioFile(ui->waveFileListWidget->item(itemIndex)->text());
    }
}

void Dialog::playAudioFile(QString fileName)
{
    QSound::play(fileName);
}

void Dialog::on_clearTraceButton_clicked()
{
    ui->traceDataTextEdit->clear();
}

void Dialog::on_clearAudioPushButton_clicked()
{
    if (QMessageBox(QMessageBox::Question, APP_TITLE, "Do you also want to delete the WAV files on disk?", QMessageBox::Yes | QMessageBox::No).exec() == QMessageBox::Yes)
    {
        for (int i = 0; i < ui->waveFileListWidget->count(); i++)
        {
            QFile::remove(ui->waveFileListWidget->item(i)->text());
        }
    }

    ui->waveFileListWidget->clear();
}

void Dialog::on_timeout()
{
    pressingKey = false;
    releasingKey = true;
    repaint();
}

void Dialog::resetTest()
{
    currentKeyIndex = 0;
    currentKeyCode = 0;
    pressingKey = false;
    releasingKey = false;
    testResult = true;
}

void Dialog::on_identifyButton_clicked()
{
    if (connected)
    {
        protocolHandler->sendIdentifyPacket(serialHandler);
        postTrace("Identify Request Sent");
    }
}

void Dialog::on_pairButton_clicked()
{
    if (connected)
    {
        protocolHandler->sendPairingPacket(serialHandler);
        postTrace("Pairing Request Sent");
    }
}

void Dialog::on_refreshPortsButton_clicked()
{
    // Cancel current connection
    on_comPortComboBox_currentIndexChanged(0);
    refreshPorts();
}

void Dialog::refreshPorts()
{
    connected = false;
    targetHeartbeat = false;
    controllerHeartbeat = false;
    ui->targetHeartbeatLabel->setText("Target Heartbeat:");
    ui->controllerHeartbeatLabel->setText("Controller Heartbeat:");

    // Clear our local info list, and the combo box then update
    serialPortInfoList.clear();
    ui->comPortComboBox->clear();
    ui->comPortComboBox->addItem("Select a port...");
    serialPortInfoList = QSerialPortInfo::availablePorts();
    foreach (const QSerialPortInfo &info, serialPortInfoList) {
        ui->comPortComboBox->addItem(info.portName());
    }
}

void Dialog::on_comPortComboBox_currentIndexChanged(int index)
{
    if (connected)
    {
        // Stop all streams in reverse order
        audioHandler->stopStream();
        protocolHandler->stopStream();
        serialHandler->stopStream();
        connected = false;
        targetHeartbeat = false;
        controllerHeartbeat = false;
        ui->targetHeartbeatLabel->setText("Target Heartbeat:");
        ui->controllerHeartbeatLabel->setText("Controller Heartbeat:");
    }

    if (index > 0)
    {
        ui->comPortComboBox->setItemText(0, "Close connection");

        // Try to connect to the serial port, account for null first item
        if (!serialHandler->startStream(&(serialPortInfoList.at(index-1))))
        {
            ui->comPortComboBox->setCurrentIndex(0);
            QMessageBox(QMessageBox::Information, APP_TITLE, "Error connecting to serial port, please try again", QMessageBox::Ok).exec();;
            return;
        }

        // Start the protocol handler
        protocolHandler->startStream();

        // Try to connect to the audio out channel
        if (!audioHandler->startStream())
        {
            ui->comPortComboBox->setCurrentIndex(0);
            QMessageBox(QMessageBox::Information, APP_TITLE, "Error connecting to audio output channel, please try again", QMessageBox::Ok).exec();

            // Stop our previously started streams
            protocolHandler->stopStream();
            serialHandler->stopStream();
            return;
        }

        connected = true;
    }
    else
    {
        ui->comPortComboBox->setItemText(0, "Select a port...");
    }
}

void Dialog::on_codecTypeComboBox_currentIndexChanged(const QString &codecTypeString)
{
    if (connected)
    {
        audioHandler->stopStream();
    }
    this->audioHandler->setCodecValues(codecTypeString);
    if (connected)
    {
        audioHandler->startStream();
    }
}

void Dialog::on_quickSetButton_clicked()
{
#ifdef __linux__
    QuickSetDialog qsd;
    qsd.setModal(true);
    qsd.exec();
#endif //__linux__
}

