/**
 * @file        qs_interface.h
 * @brief       the interface functions for the Quickset library
 * 
  */

/* 
 * Copyright 2010 Universal Electronics Inc. All Rights
 * Reserved.
 * 
 * NOTICE:  You are permitted to use this file only in
 * accordance with the terms of the Universal Electronic Inc.
 * license agreement under which it was obtained.
 * Redistribution, publication, or distribution in any form is
 * not permitted without a separate license.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT OWNER(S) BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 */



#ifndef _QS_IF_H_
#define _QS_IF_H_

#ifdef __cplusplus
extern "C"
{
#endif

#include "ueic_defs.h"
#include "ueic_status.h"

/* *********************************************************************** */
/* Version Access Functions                                                */
/* *********************************************************************** */

/**
 * @brief       Gets the Quickset Embedded library's revision number
 * @ingroup    version_utils
 * @return     An integer value of the current Quickset Embedded
 *             library's major revision number
 * @since      1.4
 * 
 * The Quickset Embedded library's revision system is of the form 
 * [MAJOR].[MINOR].[MILESTONE] where MAJOR is incremented with
 * the addition of major features, MINOR is incremented with
 * the addition of minor features and bug fixes, and MILESTONE is
 * incremented whenever an official build is created.  In
 * addition, release candidates are numbered during a release
 * process.  This function returns the VERSION value from this
 * system.
 */
LIBQS_API
int             QS_getRevMajor(void);


/**
 * @brief       Gets the Quickset Embedded library's revision
 * minor number
 * @return     An integer value of the current Quickset Embedded
 *             library's minor revision number
 * @since      1.4
 * 
 */
LIBQS_API
int             QS_getRevMinor(void);

/**
 * @brief       Gets the Quickset Embedded library's milestone number
 * @ingroup    version_utils
 * @return     An integer value of the current Quickset Embedded library's 
 *             milestone number
 * @since      1.0
 * 
 * The Quickset Embedded library's revision system is of the form 
 * [VERSION].[MILESTONE].[BUILD] where VERSION is incremented with the addition 
 * of major features, MILESTONE is incremented with the addition of minor 
 * features and bug fixes, and BUILD is incremented whenever an official build 
 * is created.  In addition, release candidates are numbered during a release 
 * process.  This function returns the MILESTONE value from this system.
 */
LIBQS_API
int             QS_getMilestone(void);


/**
 * @brief       Gets a string representation of the Quickset Embedded library's 
 *             revision
 * @return     A constant string of the form
 *             "v[MAJOR].[MINOR].[MILESTONE]" if the library is
 * @since      1.4
 * @attention  This function returns a constant string
 *             representation of this libary's revision. DO NOT
 *             FREE THE RETURNED STRING.
 *
 */
LIBQS_API
QS_name_t       QS_getVersionString(void);



/**
 * @brief       Gets a string representation of the Quickset
 *             Embedded Loca Database's name and revision.
 * 
 * @return     A constant string of the local database name and
 *             version
 * 
 * @since      1.8
 * @attention  This function returns a constant string of the
 *             name and version of the local database. DO NOT
 *             FREE THE RETURNED STRING.
 *             Only uses this API after successfully calling the
 *             QS_start API.
 *
 */
LIBQS_API
QS_name_t       QS_getLocalDBVersionString(void);



/**
 * @brief        starts a Quickset session; initializes local
 *               database if not alreday done so. Must be
 *               invoked before all other methods to be valid at
 *               the beginning or after QS_stop.
 * 
 * @param config the path to the Quickset configuration file.
 * 
 * @return       QS_ok if Quickset starts properly; if QS_fail
 *               use QS_get_last_error_code to get the actual
 *               error number.
 */
LIBQS_API
QS_status_t     QS_start(QS_name_t config);

/**
 * @brief        stops the Quickset session, frees all the
 *               resources used in the Quickset session.
 *              
 * @return       QS_ok if Quickset stops properly; if QS_fail
 *               use QS_get_last_error_code to get the actual
 *               error number.
 */
LIBQS_API
QS_status_t     QS_stop(void);


/**
 * @brief        If Online is to be enabled through a third party
 *               server then this API has to be called before
 *               QS_online_start can be called.
 *               QS_start(...)
 *               ...
 *               QS_online_set_registration_key(...)
 *               ...
 *               QS_online_start(...)
 *               
 * @param ueiRc  registration key used to access the online 
 *               webservices.
 * @return      ueic_success if call is successful. Otherwise
 *              use QS_get_last_error_code and
 *              QS_get_last_error_name to get
 *               the error code and its description.
 */
LIBQS_API
ueic_status_t   QS_online_set_registration_key(QS_name_t ueiRc);


/**
 * @brief      Online database access requires an unique id to
 *             authenticate * with the webservices. This API
 *             provides for a method to set the unique id for a
 *             session. This method needs to be called before
 *             the QS_online_start method is invoked. Eg..
 *
 *               QS_start(...)
 *               ...
 *               QS_online_set_unique_id(...)
 *               ...
 *               QS_online_start(...)
 *
 *
 *              This id can also be set through configuration file by defining
 *              the variable "userid". Value set through this API takes
 *              precedence in case of being set both ways.
 * Parameter:   uniqe_id could be MAC address or any other such
 *              ID's which can uniquely identify the device.
 * @return      ueic_success if call is successful. Otherwise
 *              use QS_get_last_error_code and
 *              QS_get_last_error_name to get
 *               the error code and its description.
 *
 * */
LIBQS_API
ueic_status_t   QS_online_set_unique_id(const char *unique_id);


/**
 * @brief      In order to access brands, models and codesets of 
 *             particular region we need to set region code This
 *             API provides a method to set the region code at
 *             run time. Below are the region codes of regions
 *             which needs to be set by apllication
 *		    0 - All Region
 *               1 - USA
 *               2 - Europe
 *               3 - Middle East
 *               4 - Asia Pacific
 *               5 - latin America
 *               6 - Oceania
 *               7 - Japan   
 * Parameter:   region_code : code for that perticular region
 * @return      ueic_success if call is successful. Otherwise
 *              use QS_get_last_error_code and
 *              QS_get_last_error_name to get
 *               the error code and its description.
 *
 * */
LIBQS_API
ueic_status_t   QS_online_set_region_code(QS_u8 region_code);

/**
 * @brief        starts an Online Quickset session; Initiates contact
 *               with the online webservices. Must be
 *               invoked before all other methods related to online
 *               retrieval methods to be valid.
 *               A successful QS_start is needed before QS_online_start
 *               can be called.
 * 
 * @param config the path to the Quickset configuration file.
 * 
 * @return       ueic_success if Online Quickset starts properly.
 *               Use QS_get_last_error_code and QS_get_last_error_name
 *               to get the error code and its description.
 */
LIBQS_API
ueic_status_t   QS_online_start(QS_name_t config);

/**
 * @brief        Retrieves device types in the database.
 *              
 * @return       the count of total found device types in the
 *               database. If return -1, use API
 *               QS_get_last_error_code to get the ueic_status_t
 *               error code.
 *  */
LIBQS_API
QS_size_t       QS_retrieve_device_types(void);

/**
 * @brief        Gets the device type name from the
 *               retrieved results of method
 *               QS_retrieve_device_types.
 * 
 * @param index  the index of type needed. Valid index must be
 *               between 0 and total types - 1.
 * 
 * @return       const string of the device type name. If NULL,
 *               use QS_get_last_error_code to get the actual
 *               error number.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_get_device_type_name(QS_index_t index);


/**
 * @brief        Retrieves the brand names with the selected
 *               device type and a prefix.
 * 
 * @param type   the selected device type name. Valid name can
 *               be received from the returns of
 *               the QS_get_device_type_name method.
 * 
 * @param prefix the prefix of the brand names wanted
 * 
 * @return       the count of total found matching brand names
 *               in the database.
 *               If return -1, use API QS_get_last_error_code to
 *               get the ueic_status_t error code.
 */
LIBQS_API
QS_size_t       QS_retrieve_brands(QS_name_t   type, QS_name_t prefix);

/**
 * @brief        Retrieves the brand names with the selected
 *               device type and a prefix from online database.Depending upon
 *               the brands_data_flag data compaction is done with the current
 *               data. 
 *
 * @param type   the selected device type name. Valid name can
 *               be received from the returns of
 *               the QS_get_device_type_name method.
 * 
 * @param prefix the prefix of the brand names wanted
 * 
 * @param brands_data_flag  this flag defines as to how to treat
 *                          the data obtained from the online database.
 *
 * @return       The enum of the error code of the last fail
 *               method.
 */
LIBQS_API
ueic_status_t   QS_online_retrieve_brands(QS_name_t   type, QS_name_t prefix, QS_data_flag_t brands_data_flag);

/**
 * @brief        Gets the brand name string from the retrieved
 *               results of method QS_retrieve_brands.
 * 
 * @param index  the index of the brand name needed. Valid index
 *               must be between 0 and total found brands - 1.
 *               A special index -1 can be passed in to get the
 *               "next" brand name.
 * 
 * @return       const string of the brand name. If NULL,
 *               use QS_get_last_error_code to get the actual
 *               error number.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_get_brand_name(QS_index_t  index);

/**
 * @brief        Gets the provider name string from the retrieved
 *               results of method QS_online_retrieve_providers.
 * 
 * @param index  the index of the provider name needed. Valid index
 *               must be between 0 and total found brands - 1.
 *               A special index -1 can be passed in to get the
 *               "next" provider name.
 * 
 * @return       const string of the provider name. If NULL,
 *               use QS_get_last_error_code to get the actual
 *               error number.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_get_provider_name(QS_index_t  index);

/**
 * @brief        Gets the total number of the brand names from
 *               retrieved results of method QS_retrieve_brands.
 * 
 * @return       the count of total found matching brand names
 *               in the database. If -1, use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_size_t       QS_get_brands_total(void);


/**
 * @brief        Gets the total number of the provider names from
 *               retrieved results of method QS_online_retrieve_providers.
 * 
 * @return       the count of total found matching provider names
 *               in the database. If -1, use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_size_t       QS_get_providers_total(void);


/**
 * @brief        Retrieves the model names with the selected
 *               brand index and a prefix.
 * 
 * @param index  the index of the selected device brand name.
 *               Valid index is between 0 and total found brands
 *               -1.
 * 
 * @param prefix the prefix of the model names wanted
 * 
 * @return       the count of total found matching model names
 *               in the database. If -1, use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_size_t       QS_retrieve_models(QS_index_t index, QS_name_t   pre_fix);

/**
 * @brief        Retrieves the model names with the selected
 *               brand index and a prefix from online database.
 *               Depending upon the models_data_flag data compaction is 
 *               done with the current data. 
 * 
 * @param index  the index of the selected device brand name.
 *               Valid index is between 0 and total found brands
 *               -1.
 * 
 * @param prefix the prefix of the model names wanted. Prefix has to be 
                 NON-NULL and a minimum of 3 characters.
 * 
 * @param models_data_flag  this flag defines as to how to treat
 *                          the data obtained from the online database.
 *
 * @return       The enum of the error code of the last fail
 *               method.
 */
LIBQS_API
ueic_status_t   QS_online_retrieve_models(QS_index_t index, QS_name_t  pre_fix, QS_data_flag_t models_data_flag);


/**
 * @brief        Retrieves compact codeset list for given brand 
 *               index and model search string. The search
 *               result list can be modified using
 *               fuzzy_threshold value. All the searched model
 *               names must have fuzzy values great than or
 *               equal to fuzzy_threshold.
 * 
 * @param brand_index  the index of the selected device brand 
 *               name. Valid index is between 0 and total found
 *               brands - 1.
 * 
 * @param model_prefix the prefix of the model names wanted. 
 *  
 * @param fuzzy_threshold  Minimum fuzzy threshold value for 
 *                         model search.
 *
 * @return      Total number of codesets. -1 in case of any 
 *              error.
 */
LIBQS_API
int             QS_retrieve_codesets_by_modelSearch(QS_index_t brand_index, QS_name_t model_prefix, QS_index_t fuzzy_threshold);


/**
 * @brief        Gets the model name string from the retrieved
 *               results of last invoked method
 *               QS_retrieve_models.
 * 
 * @param index  the index of model needed. Valid index must be
 *               between 0 and total found models - 1. A special
 *               index -1 can be passed in to get the "next"
 *               model name.
 * 
 * @return       const string of the model name. If NULL,
 *               use QS_get_last_error_code to get the actual
 *               error number.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_get_model_name(QS_index_t  index);


/**
 * @brief        Gets the model's fuzzy value from the retrieved
 *               results of last invoked method
 *               QS_retrieve_models.
 * 
 * @param index  the index of model needed. Valid index must be
 *               between 0 and total found models - 1. A special
 *               index -1 can be passed in to get the "next"
 *               model fuzzy value.
 * 
 * @return       The fuzzy value of the model name. If NULL, use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_index_t      QS_get_model_fuzzy_value(QS_index_t  index);


/**
 * @brief        Gets the total count of the model names from
 *               the retrieved results of the method
 *               QS_retrieve_models.
 * 
 * @return       the count of total found matching model names
 *               in the database. If -1, use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_size_t       QS_get_models_total(void);


/**
 * @brief        Retrieves the codeset with the selected
 *               model index.
 * 
 * @param model_idx  the index of the selected mode name. Valid
 *               index is between 0 and total found models
 *               -1.
 * 
 * @return       the count of total found matching codeset names
 *               in the database. If -1, use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_size_t       QS_retrieve_codesets(QS_index_t  model_idx);


/**
 * @brief        Retrieves the codeset with the selected
 *               model index from online database.
 *               Depending upon the codesets_data_flag data compaction is 
 *               done with the current data. 
 * 
 * @param index  the index of the selected mode name. Valid
 *               index is between 0 and total found models
 *               -1.
 * 
 * @param codesets_data_flag  this flag defines as to how to treat
 *                            the data obtained from the online database.
 *
 * @return       The enum of the error code of the last fail
 *               method.
 */
LIBQS_API
ueic_status_t   QS_online_retrieve_codesets(QS_index_t  index, QS_data_flag_t codesets_data_flag);


/**
 * @brief        Retrieves the codeset with the selected
 *               brand index. This method enables the brand only
 *               search.
 * 
 *               It needs be be called right after
 *               QS_retrieve_brands, or
 *               QS_online_retrieve_brands.
 * 
 * @param brand_index  the index of the selected brand name.
 *               Valid index is between 0 and total found brands
 *               -1.
 * 
 * @return       the count of total found matching codeset names
 *               in the database. If -1, use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_size_t       QS_retrieve_codesets_by_brand(QS_index_t  brand_index);


/**
 * @brief        Retrieves the codeset with the selected
 *               brand index. This method enables the brand only
 *               search. Data is retrieved from online database.
 *               Depending upon the codesets_data_flag data compaction is 
 *               done with the current data. 
 * 
 * @param index  the index of the selected brand name. Valid
 *               index is between 0 and total found brands
 *               -1.
 * 
 * @param codesets_data_flag  this flag defines as to how to treat
 *                          the data obtained from the online database.
 *
 * @return       The enum of the error code of the last fail
 *               method.
 */
LIBQS_API
ueic_status_t   QS_online_retrieve_codesets_by_brand(QS_index_t  index, QS_data_flag_t codesets_data_flag);

/**
 * @brief        Retrieves the codeset with the selected
 *               provider index. This method enables the brand only
 *               search. Data is retrieved from online database.
 *               Depending upon the codesets_data_flag data compaction is 
 *               done with the current data. 
 * 
 * @param index  the index of the selected provider name. Valid
 *               index is between 0 and total found brands
 *               -1.
 * 
 * @param codesets_data_flag  this flag defines as to how to treat
 *                          the data obtained from the online database.
 *
 * @return       The enum of the error code of the last fail
 *               method.
 */
LIBQS_API
ueic_status_t   QS_online_retrieve_codesets_by_provider(QS_index_t  index, QS_data_flag_t codesets_data_flag);

/**
 * @brief        Gets the codeset name string from the retrieved
 *               results of the method QS_retrieve_codesets.
 * 
 * @param index  the index of codeset needed. Valid index must
 *               be between 0 and total found codeset - 1. A special
 *               index -1 can be passed in to get the "next"
 *               codeset name.
 * 
 * @return       const string of the codeset name. If NULL,
 *               use QS_get_last_error_code to get the actual
 *               error number.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_get_codeset_name(QS_index_t  index);


/**
 * @brief        Gets the total count of the codeset names from
 *               the retrieved results of the method
 *               QS_retrieve_codesets.
 * 
 * @return       the count of total found codeset names
 *               in the database. If -1, use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_size_t       QS_get_codeset_total(void);


/**
 * @brief        Sends the codeset binary code to the 2 way
 *               enabled remote control devices
 * 
 * @param index  the index of the selected codeset. Valid index
 *               must be between 0 and total found codeset
 *               - 1.
 * 
 * @param mode   the mode key on the target remote to be
 *               programmed.
 * 
 * @return       QS_ok if programed the remote successfully.
 *               QS_fail otherwise, and use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_status_t     QS_send_codeset(QS_index_t  index, remote_mode_t mode);


/**
 * @brief        Sends the codeset binary code to the 2 way
 *               enabled remote control devices by codeset name.
 * 
 * @param name  the name of the codeset. such as "S0011"
 * 
 * @param mode   the mode key on the target remote to be
 *               programmed.
 * 
 * @return       QS_ok if programed the remote successfully.
 *               QS_fail otherwise, and use
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_status_t     QS_send_codeset_by_name(QS_name_t name, remote_mode_t mode);



/**
 * @brief           retrieves codeset in binary form. This API is
 *                  enabled for remote products only. For Blaster
 *                  products QS_send_codeset/QS_send_codeset_by_name
 *                  is to be used.
 *
 * @param name         the name of the codeset. such as "S0011"
 * 
 * @param buf_out      returned buffer containg codeset in binary form  
 *               
 * @param buf_sz_out   size of the returned buffer
 * 
 * @return       0 if success
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_s32
QS_get_codeset_binary(QS_name_t name, QS_u8 **buf_out, QS_s32 *buf_sz_out);

/**
 * @brief           retrieves codeset in binary form from the 
 *                  specified database file.
 *
 * @param name         the name of the codeset. such as "S0011"
 *  
 * @param dbfilename      name of the database. 
 *  
 * @param buf_out      returned buffer containg codeset in binary form  
 *               
 * @param buf_sz_out   size of the returned buffer
 * 
 * @return       0 if success
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_s32
QS_get_codeset_binary_from_file( QS_name_t in_config_file_path, QS_name_t name, QS_name_t dbfilename, QS_u8 **buf_out, QS_s32 *buf_sz_out);

/**
 * @brief           retrieves codeset in binary form from online
 *                  database.
 *
 * @param name         the name of the codeset. such as "S0011"
 * 
 * @param buf_out      returned buffer containg codeset in binary form  
 *               
 * @param buf_sz_out   size of the returned buffer
 * 
 * @return       0 if success QS_get_last_error_name to get the 
 *               actual error name amd
 *               QS_online_get_last_error_name to get the
 *               network error.
 */
LIBQS_API
ueic_status_t
QS_online_get_codeset_binary(QS_name_t name, QS_u8 **buf_out, QS_s32 *buf_sz_out);


/**
 * @brief           retrieves Aircon codeset in binary form. 
 *
 * @param name         the name of the codeset. 
 *  
 * @param name_sz      the length of codeset name. 
 * 
 * @param buf_out      returned buffer containg Aircon (decrypted) codeset in binary form  
 *               
 * @param buf_sz_out   size of the returned buffer
 * 
 * @return       0 if success
 *               QS_get_last_error_code to get the actual error
 *               number.
 */
LIBQS_API
QS_s32
QS_get_air_con_codeset_binary(QS_name_t name, QS_s32 name_sz, QS_u8 **buf_out, QS_s32 *buf_sz_out);


/**
 * @brief        Gets the error code enum number for the last
 *               failed method.
 * 
 * @return       The enum of the error code of the last fail
 *               method.
 */
LIBQS_API
ueic_status_t    QS_get_last_error_code(void);

/**
 * @brief        Returns the error code from querying the online 
 *               data base. This API is to be used when ueic_status_t
 *         QS_get_last_error_code      returned is ueic_onlinedb_query_error.
 *
 * @return       The error code from querying the online database.
 *               
 */
LIBQS_API
int              QS_online_get_last_error_code(void);

/**
 * @brief        Gets the error code string for the last
 *               failed method.
 * 
 * @return       The string version of error code of the last
 *               fail method.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_get_last_error_name(void);

/**
 * @brief        Returns the error description from querying the online 
 *               data base. This API is to be used when ueic_status_t
 *               returned is ueic_onlinedb_query_error.
 *
 * @return       The string version of online database query error. 
 *               
 *               
 */
LIBQS_API
QS_name_t       QS_online_get_last_error_name(void);



/**
 * @brief   Request is made to cancel Online query. This Function
 *             MUST be called from separate thread (Not from the thread
 *             which is giving Online Request to the Online Server).
 *
 * @return  QS_ok if successful.
 *               QS_fail otherwise, and use
 *               QS_get_last_error_code to get the actual error
 *               number.               
 *               
 */
LIBQS_API
QS_status_t     QS_online_cancel_request(void);

/* methods for options */

/**
 * @brief        Sets the minimum allowed fuzzy logic
 *               threshold. This only affects the retrieve
 *               model method. All the returned model names must
 *               have fuzzy values great than or equal to the
 *               minimum threshold.
 * 
 * @param threshold valid threshold needs to be between 0 to
 *                   100.
 * 
 * @return       QS_ok if succeed, QS_fail if parameter
 *               threshold is out side the valid boundaries.
 */
LIBQS_API
QS_status_t     QS_Option_set_minimum_fuzzy_threshold(QS_index_t threshold);

LIBQS_API
QS_index_t      QS_Option_get_minimum_fuzzy_threshold(void);



/**
 * @brief        Sets the fuzzy logic search properties. In can
 *               be combined with multiple properties with &
 *               operator. Check ueic_defs.h for allowed
 *               properties names.
 *
 * @param fuzzy  Fuzzy logic search properties.
 * 
 * @return       QS_ok if succeed, QS_fail if parameter has
 *               invalid combination
 * 
 *  */
LIBQS_API
QS_status_t     QS_Option_fuzzy_properties_set(ueic_fuzzyFlags_t fuzzy);

/**
 * @brief        Gets currently configured fuzzy logic search
 *               properties. Check ueic_defs.h for possible
 *               values of fuzzy properties.
 * 
 * @return       current fuzzy property value. This value can be 
 *               result of '|' operation of multiple
 *               ueic_fuzzyFlags_t values.
 * 
 *  */
LIBQS_API
ueic_fuzzyFlags_t QS_Option_fuzzy_properties_get();


/* methods for retrieving current status */


/**
 * @brief        Gets the last selected device type by
 *               the method QS_retrieve_brands.
 * 
 * @return       The const string of type name.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_status_device_type(void);

/**
 * @brief        Gets the last selected brand name by the
 *               method QS_retrieve_models.
 * 
 * @return       The const string of brand name.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_status_brand_name(void);

/**
 * @brief        Gets the last selected model name by
 *               the method QS_retrieve_codesets.
 * 
 * @return       The const string of the model name.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_status_model_name(void);

/**
 * @brief        Gets the last sent codeset name by
 *               the method QS_send_codeset.
 * 
 * @return       The const string of the codeset name.
 */
/*@null@*/
LIBQS_API
QS_name_t       QS_status_codeset_sent(void);

/**
 * @brief       Get the current query mode. It is only valid
 *              after successful QS_start call.
 * 
 * @return      QS_query_normal means all supported query
 *               methods are enabled.
 *              QS_query_no_model means model data querying is
 *               disabled.
 *              QS_query_invalid means quickset is not start
 *              properly, call QS_start first.
 *
 * */
LIBQS_API
QS_query_mode_t QS_query_mode_get(void);


/**
 * @brief       Reset the query mode to QS_query_normal, and clear
 *              the codeset download counter to 0. It is only valid
 *              after successful QS_start call.
 *
 * Parameter:   reset_flag has 2 possible values,
 *              QS_query_reset_normal: reset the counter
 *                 and set the download threshold to normal (100)
 *              QS_query_reset_test  : reset the counter
 *                  and set the download threshold to test
 *                  level(5)
 *         
 * @return      QS_ok the query mode is set the QS_query_normal
 *              QS_fail, unable to set the query mode to
 *              QS_query_normal. This only happens when there is
 *              a hardware failure on the blaster chip.
 * */
LIBQS_API
QS_status_t     QS_query_mode_reset(QS_query_mode_reset_t reset_flag);


/**
 * @brief        use edid data to find matching codeset(s) 
 *               for TV devices.
 * 
 * @ingroup      Automatic Discovery
 * 
 * @since        2.6
 *
 * @param edid_buf the buffer contains the edid data.
 *
 * @param edid_buf_sz  the edid data size, it should be >= 128.
 *
 * @return      Number of codesets found that matches the 
 *              edid data, 0 if none found, -1 in case of
 *              any error.
 */

LIBQS_API
QS_size_t       QS_EDID_retrieve_codesets(const QS_u8 *edid_buf_org,
                                          QS_size_t edid_buf_sz);



/**
 * @brief        use edid data to find matching codeset(s) 
 *               for TV devices thru online lookup.
 * 
 * @ingroup      Automatic Discovery
 * 
 * @since        2.6
 *
 * @param edid_buf the buffer contains the edid data.
 *
 * @param edid_buf_sz  the edid data size, it should be >= 128.
 *
 * @return       The result of the call.  To determine the number of
 *               codesets found that match the EDID data call the
 *               QS_get_codeset_total() API.
 */
LIBQS_API
QS_status_t     QS_online_EDID_retrieve_codesets(QS_u8 *edid_buf,
                                                 QS_size_t edid_buf_sz);


/**
 * @brief        use CEC information to find matching codesets
 *               for TV devices.
 * 
 * @ingroup      Automatic Discovery
 * 
 * @since        2.6
 *
 * @param cecVendorID  the cec vender ID (3 bytes), and it is 
 *                     required for TV; optional for Audio.
 *
 * @param cecLogicAddr the logic address for the device (0 ~ 14)
 *
 * @param osdString    the OSD name of device, and it is 
 *                     optional for TV; required for Audio.
 *
 * @return       Number of codesets found that matches the 
 *               cec data, 0 if none found, -1 in case of
 *               any error.
 */
LIBQS_API
QS_size_t       QS_CEC_retrieve_codesets(QS_u32      cecVendorID,
                                         QS_u16      cecLogicAddr,
                                         QS_name_t   osdString);

/**
 * @brief        This function only can be invoked after a
 *               successful QS_EDID_retrieve_codesets or
 *               QS_CEC_retrieve_codesets call.
 * 
 * @ingroup      Automatic Discovery
 * 
 * @since        2.6
 *
 * @return       The a const char points to the name string of
 *               the TV brand found in CEC/EDID lookup
 *
 */
LIBQS_API
QS_name_t       QS_EDID_CEC_get_brand_name(void);

/**
 * @brief        Retrieves the provider names with the selected regionCode and
 *               location.  Depending upon the data_flag data compaction is
 *               done with the current data.
 *
 * @param regionType   The type of region being provided.
 * 
 * @param regionCode   The region to search.
 * 
 * @param location     The location of the region, currently only the location
 *                     "United States" is supported.  Providing a
 *                     NULL value = "United States".
 *
 * @param data_flag    This flag defines as to how to treat
 *                     the data obtained from the online database.
 *
 * @return       Function result code.
 */
LIBQS_API
ueic_status_t
QS_online_retrieve_providers(QS_region_code_type_t regionType,
                             QS_name_t regionCode,
                             QS_name_t location,
                             QS_data_flag_t data_flag);

/* Optimized Setup Maps */

/**
 * @brief        This API reports the testing result of a tested Key.
 * 
 * @ingroup     OptimizedSetupMaps
 * 
 * @since        2.2
 *
 * @param response: WORKED(0) If IR Signal of the key worked 
 *               fine with device.  
 *               DID_NOT_WORK(1) If IR Signal of the key did not 
 *               work with the device. For more information
 *               see enum QS_osm_user_response_t.
 *
 * @return       kResult_GET_NEXT (1) There are more keys to test, 
 *               continue testing IR Keys.
 *               kResult_SUCCESS (2) No More Keys to test. This
 *               attempt is successful .
 *               kResult_FAILED (3) Failed to get codeset from
 *               search, restart whole process. This attempt failed.
 *               kResult_ERROR (-1) this is returned in case of error.
 *               For more information see QS_osm_test_result_response_t 
 *               enum.
 */
LIBQS_API
QS_osm_test_result_response_t QS_osm_report_test_result(
    QS_osm_user_response_t response);

/**
 * @brief       Returns total number of IR keys can be used
 *              for testing the Device. The quickset session
 *              loads all related IR codesets to the test list
 *              internally.
 * 
 * @ingroup    OptimizedSetupMaps
 * 
 * @since       2.2
 *
 * @param brand_index: brand to be tested
 * 
 * @param unused: Unused, set to NULL.
 *
 * @return      Total key can be used for testing this IR. If 1
 *              is returned, means only one possible IR code is
 *              found.
 * 
 */
LIBQS_API
QS_size_t QS_osm_get_test_keys_total(QS_index_t brand_index,
                                     QS_name_t  unused);
/**
 * @brief       Returns all of the active codeset names being 
 *              tested.
 * 
 * @ingroup     OptimizedSetupMaps
 * 
 * @since       3.0
 *
 * @param       out_name_array. The array of names. This memory 
 *              should not be freed by the caller.
 *           
 * 
 * @param       out_name_array_count. The number of elements in 
 *              the out_name_array. If zero, then an error
 *              ocurred.
 */
LIBQS_API
void QS_osm_get_all_active_codeset_names( QS_name_t **out_name_array, QS_u32 *out_name_array_count );

/**
 * @brief       Marks the input codesets as valid, and all other 
 *              codesets as invalid. This call should be made
 *              directly after QS_osm_get_test_keys_total().
 * 
 * @ingroup     OptimizedSetupMaps
 * 
 * @since       2.4
 *
 * @param in_valid_codesets     Array of valid codeset names.
 * 
 * @param in_valid_codesets_count   The number of codesets in 
 *                                  the in_valid_codesets array.
 * 
 */
LIBQS_API
void QS_osm_pre_validate_codesets(const char **in_valid_codesets,
                                  QS_index_t in_valid_codesets_count);

/**
 * @brief       Get the first key to test
 * 
 * @ingroup    OptimizedSetupMaps
 * 
 * @since       2.2
 *
 * @return      On normal key ID return if still keys to test;
 *              return kKey_INVALID if any error happens; use
 *              QS_get_last_error_code to get the actual error
 *              number.
 */
LIBQS_API
QS_osm_key_t QS_osm_get_first_test_key(void);

/**
 * @brief       Get the next key to test
 * 
 * @ingroup    OptimizedSetupMaps
 * 
 * @since       2.2
 *
 * @return      On normal key ID return kKey_INVALID if any error
 *              happens; use QS_get_last_error_code to get the
 *              actual error number.
 */
LIBQS_API
QS_osm_key_t QS_osm_get_next_test_key(void);

/**
 * @brief       Return the name of the current codeset being tested.
 * 
 * @ingroup     OptimizedSetupMaps
 * 
 * @since       2.2
 * 
 * @return      The current codeset name.
 */
LIBQS_API
QS_name_t QS_osm_get_current_codeset_name(void);

/**
 * @brief       Return the total number of codesets being
 *              tested.  The current number of codesets tested divided
 *              by the total number of codesets being tested yields the
 *              completion percentage.
 * 
 * @ingroup     OptimizedSetupMaps
 * 
 * @since       2.2
 * 
 * @return      The total number of codesets being tested.
 */
LIBQS_API
int QS_osm_get_codesets_total(void);

/**
 * @brief       Return the number of codesets that have been
 *              tested.  This number divided by the
 *              total number of codesets being tested yields the
 *              completion percentage.
 * 
 * @ingroup     OptimizedSetupMaps
 * 
 * @since       2.2
 * 
 * @return      The current codeset index.
 */
LIBQS_API
int QS_osm_get_codesets_tested(void);


/*
********************************************************************************
* LOGGING APIS
********************************************************************************
*/

/**
* @brief        Start the logger.
*               
* @param inLogType   The logging type.
*
* @return       Success or failure code.
*/
LIBQS_API
QS_status_t QS_log_start(QS_log_type_t inLogType);

/**
* @brief        Stop the logger.
*/
LIBQS_API
void QS_log_stop(void);

/**
* @brief        Set the logging file.  Only used when logging is started with
*               a log type of QSLOGTYPE_FILE.
*               
* @param inLogFilePath   Full path of the log file.
*
* @return       Success or failure code.
*/
LIBQS_API
QS_status_t QS_log_set_file(const char *inLogFilePath);

/**
* @brief        Get the current size of the log buffer.  Only used when
*               logging is started with a log type of QSLOGTYPE_BUFFER.
*               The log buffer size will not change as long as no Quickset
*               calls are made, so it is safe to allocate a buffer and
*               immediately call QS_log_get_buffer().
*               
* @return       The size of the log buffer.
*/
LIBQS_API
QS_u32 QS_log_get_buffer_size(void);

/**
* @brief        Get the contents of the log buffer.  The log buffer is reset
*               after calling this function.
*               
* @param outBuffer   Destination buffer for the log.  The provided buffer
*               must be at least qslog_get_buffer_size() bytes in length.
*/
LIBQS_API
void QS_log_get_buffer(char *outBuffer);

/** 
 * @brief       Print a hex dump to the logfile. 
 *  
 * @param inBuffer   Buffer to hexdump.
 * @param inSize     Size of buffer.
 */
LIBQS_API
void QS_log_hexdump( void *inBuffer, int inSize );

/**
* @brief        Log a printf-style message.
*               
* @param inFormat   Format string.  May contain printf-style % codes.
*/
LIBQS_API
void QS_log_printf(const char *inFormat, ...);

/**
* @brief        Retrieves the number of device type records in the dac9a file. 
*
* @return       Total number of type records. Negative number indicates an error. 
*/
LIBQS_API
QS_size_t QS_region_retrieve_type_count(void);

/**
* @brief        Retrieve the device type name at specified index. The index
*         must be between 0 and N-1 inclusive, where N is the result of 
*         QS_region_retrieve_type_count.
*               
* @param inIndex   Index of selected device type.
*
* @return       Name of device type.
*/
LIBQS_API
QS_name_t QS_region_retrieve_type_name(QS_u32 inIndex);

/**
* @brief        Retrieve the number of location records for the specified
*         zipcode in the dac9a file. 
*               
* @param inZipcode   Specified zipcode. 
*
* @return       Total number of location records. Negative number indicates an error.
*/
LIBQS_API
QS_size_t QS_region_retrieve_provider_count_by_zipcode(char *inZipcode);

/**
* @brief    Retrieve the number of brands in the specified region, within
*         the index of the specified device type. Both region and subregion
*         should be non-empty and valid strings of length 2 each. Note that 
*         this brand count is not compatible with the brand list and count 
*         produced by calling QS_retrieve_brands or QS_online_retrieve_brands.
*               
* @param inTypeIndex  Selected device type index.
*
* @param inRegionCode  UEI-defined code for major region. Ex. NA, U, WW, etc.
*
* @param inSubRegionCode  ISO-3166 standard country code.  
*
* @return       Total number of brands.
*/
LIBQS_API
QS_size_t QS_region_retrieve_brand_count_by_region(QS_u32 inTypeIndex,
                                                   QS_name_t inRegionCode,
                                                   QS_name_t inSubRegionCode);

/**
* @brief        Retrieve the brand name at specified index. The index must be
*         between 0 and N-1 inclusive, where N is the result of QS_region_
*         retrieve_brand_count_by_region.
*
* @param inBrandIndex  Specified brand index.
*
* @return       Name for the selected index of brand.
*/
LIBQS_API
QS_name_t QS_region_retrieve_brand_name(QS_u32 inBrandIndex);

/**
* @brief  Retrieve the codeset count of specific brand. The brand index must be
*         between 0 and N-1 inclusive, where N is the result of QS_region_
*         retrieve_brand_count_by_region.
*
* @param brand_index  Specified brand index.
*
* @return       Number of codeset of the selected index of brand.
*/
LIBQS_API
QS_size_t QS_region_retrieve_codeset_count(QS_index_t brand_index);

/**
* @brief        Retrieve the codeset name at specified index. The index must be
*         between 0 and N-1 inclusive, where N is the result of QS_region_
*         retrieve_codeset_count
*
* @param codeset_index Specified codeset index
*
* @return       Name for the selected index of codeset.
*/
LIBQS_API
QS_name_t QS_region_retrieve_codeset_name(QS_index_t codeset_index);

/**
* @brief        Retrieve the provider name at the specified index. The index must
*         be between 0 and N-1 inclusive, where N is the result of QS_region_
*         retrieve_provider_count_by_zipcode.
*               
* @param inProviderIndex  Specified provider index.
*
* @return      Name for the selected index of provider.
*/
LIBQS_API
QS_name_t QS_region_retrieve_provider_name(QS_u32 inProviderIndex);

/**
* @brief        Retrieves the total number of test keys that can be used for testing,
*         but uses a Brand or Provider name instead of a Brand index.
*
* @param inName  Name of provider or brand.
*
* @return       Total number of test keys.
*/
LIBQS_API
QS_size_t QS_osm_get_test_keys_total_by_name(QS_name_t inName);

/********************ADDED FOR IOS*******************************/
void qs_set_user_home(const char *usr);
/****************************************************************/

/**
 * @brief     Retrieve codesets based on infomation
 *            retrieved from upnp xml data buffer.
 * 
 * @ingroup   Automatic Discovery
 * 
 * @since     2.6
 * 
 * @param     upnp_xml_data.          upnp xml data buffer.
 * @param     upnp_xml_data_sz.       upnp xml data buffer size.
 * @return    Number of codesets found that matches the xml 
 *            data, 0 if none found, -1 in case of any error.
 */
LIBQS_API
QS_size_t   QS_ip_retrieve_codesets(const char  *upnp_xml_data,
                                    QS_size_t   upnp_xml_data_sz);

/**
 * @brief     Retrieve codesets based on infomation
 *            retrieved from upnp discrete data.
 * 
 * @ingroup   Automatic Discovery
 * 
 * @since     2.6
 * 
 * @param     manufacturer       manufacturer name.
 * @param     manufacturer_url   manufacturer url.
 * @param     friendly_name      friendly name.
 * @param     model_name         model name.
 * @param     model_number       model number.
 * @param     model_description  model description.
 * @return    Number of codesets found that matches the xml
 *            data, 0 if none found, -1 in case of any error.
 */
LIBQS_API
QS_size_t   QS_ip_discrete_retrieve_codesets(const char *manufacturer,
                                             const char *manufacturer_url,
                                             const char *friendly_name,
                                             const char *model_name,
                                             const char *model_number,
                                             const char *model_description);

/**
 * @brief     Retrieve CEC Vendor ID for brand name.
 * 
 * @ingroup   Automatic Discovery
 * 
 * @since     2.6
 * 
 * @param     brand_name_org     brand name
 * @return    CEC Vendor ID, or 0 if cannot find CEC Vendor ID
 */
LIBQS_API
QS_u32      QS_cec_get_VendorID(QS_name_t brand_name_org);

/**
 * @brief     Retrieve codesets based on infomation
 *            retrieved from infoframe raw data.
 * 
 * @ingroup   Automatic Discovery
 * 
 * @since     2.6
 * 
 * @param     raw_data.          infoframe raw data.
 * @param     raw_data_sz.       infoframe raw data size.
 * @return    Number of codesets found that matches the 
 *            info-frame data, 0 if none found, -1 in case of
 *            any error.
 */
LIBQS_API
QS_size_t QS_infoframe_retrieve_codesets(const char *raw_data_org,
                                         QS_size_t  raw_data_sz);

/**
 * @brief     Get manufacturer name for info-frame data.
 * 
 * @ingroup   Automatic Discovery
 * 
 * @since     2.6
 *  
 * @return    const string of the manufacturer name. If NULL, 
 *            use QS_get_last_error_code to get the actual error
 *            number.
 */
LIBQS_API
QS_name_t QS_infoframe_get_manufacturer_name(void);

/**
 * @brief     Gets the region name string for codeset retrieved
 *            from the retrieved results of the method
 *            QS_retrieve_codesets. This function only can be
 *            invoked after a successful auto discovery call.
 * 
 * @ingroup   Automatic Discovery
 * 
 * @since     2.6
 * 
 * @param     index  the index of codeset needed. Valid index
 *                   must be between 0 and total found codeset -
 *                   1. A special index -1 can be passed in to
 *                   get the "next" codeset name.
 * 
 * @return    const string of the region name. If NULL, use
 *            QS_get_last_error_code to get the actual error
 *            number.
 */
LIBQS_API
QS_name_t QS_auto_discovery_codeset_region(QS_index_t  uei_index);

/**
* @brief      Return the number of countries with province data. 
*  
* @return     The number of countries.
*/
LIBQS_API
QS_index_t QS_province_get_country_count( void );

/**
* @brief    Return the name of a country.
*  
* @param    country_index.  The index of the country.  Valid 
*           index values are from 0..N-1, where N is the
*           result of QS_province_get_country_count.
*  
* @return   Name of the country.
*/
LIBQS_API 
QS_name_t QS_province_get_country_name( QS_index_t country_index );

/**
* @brief    Return the number of provinces within a country.
*  
* @param    country_index.  The index of the country.  Valid 
*           index values are from 0..N-1, where N is the
*           result of QS_province_get_country_count.
*  
* @return   The number of provinces within the country.
*/
LIBQS_API 
QS_index_t QS_province_get_province_count( QS_index_t country_index );

/**
* @brief    Return the name of a province.
*  
* @param    province_index.  The index of the province within a 
*           country. Valid index values are from 0..N-1, where N
*           is the result of QS_province_get_province_count.
*  
* @return   The province name.
*/
LIBQS_API 
QS_name_t QS_province_get_province_name( QS_index_t province_index );

/**
* @brief    Return the number of cities within a province.
*  
* @param    country_index.  The index of the country.  Valid
*           index values are from 0..N-1, where N is the
*           result of QS_province_get_country_count.
*  
* @param    province_index.  The index of the province within a 
*           country. Valid index values are from 0..N-1, where N
*           is the result of QS_province_get_province_count.
*  
* @return   The number of cities within the province.
*/
LIBQS_API 
QS_index_t QS_province_get_city_count( QS_index_t province_index );

/**
* @brief    Return the name of a city.
*  
* @param    city_index.  The index of the city within a
*           province. Valid index values are from 0..N-1, where
*           N is the result of QS_province_get_city_count.
*  
* @return   The province name.
*/
LIBQS_API 
QS_name_t QS_province_get_city_name( QS_index_t city_index );

/**
* @brief    Return the number of providers within a city.
*
* @param    city_index.  The index of the city within a 
*           province. Valid index values are from 0..N-1, where
*           N is the result of QS_province_get_city_count.
*
* @return   The number of providers.
*/
LIBQS_API
QS_index_t QS_province_get_city_provider_count( QS_index_t city_index );

/**
* @brief    Return the name of a provider.
*
* @param    provider_index.  The index of the provider within a
*           city.  Valid index values are from 0..N-1, where
*           N is the result of
*           QS_province_get_city_provider_count.
*
* @return   The provider name.
*/
LIBQS_API
QS_name_t QS_province_get_city_provider_name( QS_index_t provider_index );

/**
* @brief    Return the number of codesets within a provider.
*  
* @param    provider_index.  The index of the provider within a 
*           city.  Valid index values are from 0..N-1, where
*           N is the result of
*           QS_province_get_city_provider_count.
*  
* @return   The number of codesets.
*/
LIBQS_API
QS_index_t QS_province_get_city_provider_codeset_count( QS_index_t provider_index );

/**
* @brief    Return the name of a codeset.
*
* @param    codeset_index.  The index of the codeset within a 
*           provider.  Valid index values are from 0..N-1, where
*           N is the result of
*           QS_province_get_city_provider_codeset_count.
*
* @return   The codeset name.
*/
LIBQS_API
QS_name_t QS_province_get_city_provider_codeset_name( QS_index_t codeset_index );

/**
* @brief    Return the number of providers within a province.
*
* @param    province_index.  The index of the province within a 
*           country. Valid index values are from 0..N-1, where N
*           is the result of QS_province_get_province_count.
*
* @return   The number of providers.
*/
LIBQS_API
QS_index_t QS_province_get_province_provider_count( QS_index_t province_index );

/**
* @brief    Return the name of a provider.
*  
* @param    provider_index.  The index of the province within a 
*           country. Valid index values are from 0..N-1, where N
*           is the result of
*           QS_province_get_province_provider_count.
*  
* @return   The provider name.
*/
LIBQS_API
QS_name_t QS_province_get_province_provider_name( QS_index_t provider_index );

/**
* @brief    Return the number of codesets within a provider.
*  
* @param    provider_index.  The index of the province within a 
*           country. Valid index values are from 0..N-1, where N
*           is the result of
*           QS_province_get_province_provider_count.
*  
* @return   The number of codesets.
*/
LIBQS_API
QS_index_t QS_province_get_province_provider_codeset_count( QS_index_t provider_index );

/**
* @brief    Return the name of a codeset.
*
* @param    codeset_index.  The index of the codeset within a
*           provider. Valid index values are from 0..N-1, where
*           N is the result of
*           QS_province_get_province_provider_codeset_count.
*
* @return   The codesets name.
*/
LIBQS_API
QS_name_t QS_province_get_province_provider_codeset_name( QS_index_t codeset_index );


#ifdef _IR_LEARNER_
/**
 * @brief   API converts raw data lernt from IR blaster toWaveform  
 *          format. Raw data learnt from the blaster should be  
 *          Passed to the API. Converted data in waveform format
 *          will be returned in the output parameter
 
 *  
 * @param raw_data    Raw data learnt from IR blaster
 *  
 * @param raw_len     length or raw data learnt from blaster
 *  
 * @param generic_format	Output data in waveform format.
 *  
 * @return QS_ok on success; QS_fail otherwise. 
 */
LIBQS_API
QS_status_t  QS_uei_learner_to_irb_convertion(unsigned char *raw_data, int raw_len,  QS_irb_waveform_t *generic_format);


/**
 * @brief   API returns the toggle waveform in waveform format
 *          QS_uei_learner_to_irb_convertion should be called before   
 *          calling this API. If toggle is > 1 this API is called
 *          Toggle waveform is retrieved based on index
 *  
 * @param toggle_idx    If total toggle is n then index id (0 - n-1)
 *  
 * @param generic_format	Output data in waveform format.
 *  
 * @return QS_ok on success; QS_fail otherwise. 
 */
LIBQS_API
QS_status_t QS_uei_learner_get_toggle_waveform(int toggle_idx, QS_irb_waveform_t *toggle_waveform);

#endif

#ifdef __cplusplus
}
#endif

#endif


