//#ifdef __linux__

#include "quicksetdialog.h"
#include "ui_quicksetdialog.h"
#include "qs_interface.h"

QuickSetDialog::QuickSetDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::QuickSetDialog)
{
    ui->setupUi(this);

    // Setup QuickSet Database
    quicksetEnabled = false;
    QS_status_t qsStatus;

    // Look in the profiles directory for the conf file for QS, specifically
    // the UEI directory
    // ex: ./profiles/Universal_Electronics/<conf file>
    QDir currentDirectory = QCoreApplication::applicationDirPath();
    if (!currentDirectory.cd("profiles"))
        return;
    if (!currentDirectory.cd("Universal_Electronics"))
        return;

    // Initialize QS
    if ((qsStatus = QS_start(currentDirectory.absoluteFilePath(CONF_FILE).toStdString().c_str())) == QS_ok)
    {
        // Get the versions
        qDebug() << "\nStarting UEI QuickSet Database:";
        qDebug() << "Major Version:" << QS_getRevMajor();
        qDebug() << "Minor Version:" << QS_getRevMinor();
        qDebug() << "Milestone Version:" << QS_getMilestone();
        qDebug() << "Version String:" << QS_getVersionString();
        qDebug() << "Local DB Version String:" << QS_getLocalDBVersionString();

        // Fuzzy properties setup - unused in this demo
        //QS_Option_get_minimum_fuzzy_threshold();
        //QS_Option_set_minimum_fuzzy_threshold();
        //QS_Option_fuzzy_properties_set();

        // Get device type
        QS_size_t deviceTypeCount = QS_retrieve_device_types();
        if (deviceTypeCount >= 0)
        {
            //qDebug() << "Device Types Found = " << deviceTypeCount;
            for (QS_size_t i = 0; i < deviceTypeCount; i++)
            {
                QS_name_t deviceTypeName = QS_get_device_type_name(i);
                //qDebug() << i << ": " << deviceTypeName;
                ui->deviceTypeComboBox->addItem(deviceTypeName);
            }
        }
        else if (deviceTypeCount < 0)
        {
            qDebug() << "Error getting device brand count: " << deviceTypeCount;
        }

        quicksetEnabled = true;
        if (deviceTypeCount > 0)
        {
            ui->deviceTypeComboBox->setCurrentIndex(0);
            on_deviceTypeComboBox_currentIndexChanged(0);
        }
    }
    else
    {
        qDebug() << "\nError starting UEI QuickSet Database:" << qsStatus << QS_get_last_error_name();
    }
}

QuickSetDialog::~QuickSetDialog()
{
    if (quicksetEnabled)
    {
        QS_status_t qsStatus;
        if ((qsStatus = QS_stop()) == QS_ok)
        {
            qDebug() << "\nStopped UEI QuickSet Database";
        }
        else
        {
            qDebug() << "\nError stopping UEI QuickSet Database:" << qsStatus << QS_get_last_error_name();
        }
    }

    delete ui;
}

void QuickSetDialog::on_deviceTypeComboBox_currentIndexChanged(int index)
{
    if (quicksetEnabled)
    {
        ui->deviceBrandComboBox->clear();
        QS_size_t deviceBrandCount = QS_retrieve_brands(ui->deviceTypeComboBox->itemText(index).toStdString().c_str(),
                                                        brandPrefix.toStdString().c_str());
        if (deviceBrandCount >= 0)
        {
            //qDebug() << "Device Brands Found = " << deviceBrandCount;
            for (QS_size_t i = 0; i < deviceBrandCount; i++)
            {
                QS_name_t deviceBrandName = QS_get_brand_name(i);
                //qDebug() << i << ": " << deviceBrandName;
                ui->deviceBrandComboBox->addItem(deviceBrandName);
            }
            if (deviceBrandCount > 0)
            {
                ui->deviceBrandComboBox->setCurrentIndex(0);
            }
            else
            {
                ui->deviceBrandComboBox->setCurrentIndex(-1);
            }
        }
        else if (deviceBrandCount < 0)
        {
            qDebug() << "Error getting device brand count: " << deviceBrandCount;
        }
    }
}

void QuickSetDialog::on_deviceTypeComboBox_activated(int index)
{
    on_deviceTypeComboBox_currentIndexChanged(index);
}

void QuickSetDialog::on_deviceBrandComboBox_currentIndexChanged(int index)
{
    if (quicksetEnabled)
    {
        ui->deviceModelComboBox->clear();
        QS_size_t deviceModelCount = QS_retrieve_models(index,
                                                        modelPrefix.toStdString().c_str());
        if (deviceModelCount >= 0)
        {
            //qDebug() << "Device Models Found = " << deviceModelCount;
            for (QS_size_t i = 0; i < deviceModelCount; i++)
            {
                QS_name_t deviceModelName = QS_get_model_name(i);
                //qDebug() << i << ": " << deviceModelName;
                ui->deviceModelComboBox->addItem(deviceModelName);
            }

            if (deviceModelCount > 0)
            {
                ui->deviceModelComboBox->setCurrentIndex(0);
            }
            else
            {
                ui->deviceModelComboBox->setCurrentIndex(-1);
            }
        }
        else if (deviceModelCount < 0)
        {
            qDebug() << "Error getting device model count: " << deviceModelCount;
        }
    }
}

void QuickSetDialog::on_deviceBrandComboBox_activated(int index)
{
    on_deviceBrandComboBox_currentIndexChanged(index);
}

void QuickSetDialog::on_deviceModelComboBox_currentIndexChanged(int index)
{
    if (quicksetEnabled)
    {
        ui->deviceCodesetComboBox->clear();
        QS_size_t deviceCodesetCount = QS_retrieve_codesets(index);
        if (deviceCodesetCount >= 0)
        {
            //qDebug() << "Device Codesets Found = " << deviceCodesetCount;
            for (QS_size_t i = 0; i < deviceCodesetCount; i++)
            {
                QS_name_t deviceCodesetName = QS_get_codeset_name(i);
                //qDebug() << i << ": " << deviceCodesetName;
                ui->deviceCodesetComboBox->addItem(deviceCodesetName);
            }

            if (deviceCodesetCount > 0)
            {
                ui->deviceCodesetComboBox->setCurrentIndex(0);
                ui->setCodeButton->setEnabled(true);
            }
            else
            {
                ui->deviceCodesetComboBox->setCurrentIndex(-1);
                ui->setCodeButton->setEnabled(false);
            }
        }
        else if (deviceCodesetCount < 0)
        {
            qDebug() << "Error getting device codeset count: " << deviceCodesetCount;
        }
    }
}

void QuickSetDialog::on_deviceModelComboBox_activated(int index)
{
    on_deviceModelComboBox_currentIndexChanged(index);
}

void QuickSetDialog::on_deviceBrandPrefix_textChanged(const QString &newBrandPrefix)
{
    brandPrefix = newBrandPrefix;
    on_deviceTypeComboBox_currentIndexChanged(0);
}

void QuickSetDialog::on_deviceModelPrefix_textChanged(const QString &newModelPrefix)
{
    modelPrefix = newModelPrefix;
    on_deviceBrandComboBox_currentIndexChanged(0);
}

typedef struct {
    const QS_u8 *               ptr;
    int                         len;
    int                         key_id;
} codeset_data_reader_t;

void codeset_data_read_init(codeset_data_reader_t * reader, const QS_u8 * data, int len)
{
    reader->ptr = data;
    reader->len = len;
    reader->key_id = 1;
}

int codeset_data_read_key(codeset_data_reader_t * reader, int * key_id, const QS_u8 ** key_data)
{
    int len;

    do {
        if (reader->len < 2)
        {
            len = (reader->len == 0)? 0 : -1;
            break;
        }
        len = reader->ptr[0] | (reader->ptr[1] << 8);
        reader->len -= 2;
        reader->ptr += 2;
        if (reader->len < len)
        {
            len = -1;
            break;
        }
        if (len > 0)
        {
            if (key_id)
                *key_id = reader->key_id;
            if (key_data)
                *key_data = reader->ptr;
        }
        reader->ptr += len;
        reader->len -= len;
        reader->key_id++;
    } while (len == 0 && reader->len > 0);

    return len;
}

void dump_codeset(const char * codeset_name, const QS_u8 * codeset_data, int codeset_data_len)
{
    char name[256];
    FILE * file;

    sprintf(name, "codeset_%s.txt", codeset_name);
    if ((file = fopen(name, "w")))
    {
        codeset_data_reader_t reader;
        const QS_u8 * key_data;
        int key_id;
        int key_data_len;

        codeset_data_read_init(&reader, codeset_data, codeset_data_len);
        while ((key_data_len = codeset_data_read_key(&reader, &key_id, &key_data)) > 0)
        {
            int i;

            fprintf(file, "Key %d", key_id);
            for (i = 0; i < key_data_len; i++)
            {
                if ((i & 0xf))
                    fprintf(file, " ");
                else
                    fprintf(file, "\n");
                fprintf(file, "%02X", key_data[i]);
            }
            fprintf(file, "\n");
        }
        if (key_data_len)
            fprintf(file, "Invalid key data\n");
        fclose(file);
    }
}

void QuickSetDialog::on_setCodeButton_clicked()
{
    QS_u8 * codeset_data;
    QS_s32 codeset_data_len;

    QS_get_codeset_binary(ui->deviceCodesetComboBox->currentText().toStdString().c_str(),
        &codeset_data,
        &codeset_data_len);
    dump_codeset(ui->deviceCodesetComboBox->currentText().toStdString().c_str(),
        codeset_data,
        codeset_data_len);

    // TODO: Download action mapping from the codeset
}

//#endif //__linux__
