#include "rf4ceaudiohandler.h"
#include "dialog.h"

void offsetGain(int16s RawSample, int16s Offset, int16s Gain, int8u *Buffer);

RF4CEAudioHandler::RF4CEAudioHandler(QObject *parent, QString codecType) : QThread(parent)
{
    this->setParent(parent);

    // Setup the audio format from the device
    this->waveStreamChannels = WAVE_CHANNELS;
    this->waveStreamSampleRate = WAVE_SAMPLE_RATE;
    this->waveStreamSampleLength = WAVE_SAMPLE_SIZE;
    setCodecValues(codecType);

    // Always save streams to a WAV file
    this->saveStreamsValue = true;
    this->realTimeOutput = false;

    // Always use the default wave mapper output device
    this->audioDevice = QAudioDeviceInfo::defaultOutputDevice();

    // Setup the audio format for the output
    // Always try to stream at 48K since it's a standard rate, we will up convert
    this->audioFormat.setChannelCount(waveStreamChannels);
    this->audioFormat.setSampleRate(48000/*waveStreamSampleRate*/); // NOTE: upconverted to 48K always
    this->audioFormat.setSampleSize(waveStreamSampleLength * 8); // Convert bytes to bits
    this->audioFormat.setCodec("audio/pcm");
    this->audioFormat.setByteOrder(QAudioFormat::LittleEndian);
    this->audioFormat.setSampleType(QAudioFormat::SignedInt);

    if (!this->audioDevice.isFormatSupported(this->audioFormat))
    {
        qWarning() << "Default format not supported - trying to use nearest";
        this->audioFormat = this->audioDevice.nearestFormat(this->audioFormat);
    }

    // Create a new audio out io device, this will do the upconversion for us
    this->speakerOutput = new AudioOut48(this);
    this->audioOutput = new QAudioOutput(audioDevice, audioFormat, this);
    this->speakerOutput->start();
    this->audioOutput->start(this->speakerOutput);
}

RF4CEAudioHandler::~RF4CEAudioHandler()
{
    this->audioOutput->stop();
    this->speakerOutput->stop();
    delete this->audioOutput;
    delete this->speakerOutput;
}

void RF4CEAudioHandler::setCustomCodecSettings(QString codecString,
                                               DecodeType decodeType,
                                               int bufferSize,
                                               int16s offset,
                                               int16s gain)
{
    this->customCodecString = codecString;
    this->customAudioDecodeType = decodeType;
    this->customAudioCompressedBufferSize = bufferSize;
    this->customOffset = offset;
    this->customGain = gain;
}

bool RF4CEAudioHandler::startStream()
{
    this->start();

    return true;
}

void RF4CEAudioHandler::stopStream()
{
    // Shut down the thread and wait for completion
    runStream = false;
    // Add timeout to robustify this
    while(this->isRunning());
}

void RF4CEAudioHandler::setCodecValues(QString codecType)
{
    if (QString::compare(codecType, HARDWARE_CODEC_STRING) == 0)
    {
        this->decodeType = HARDWARE_DECODE_TYPE;
        this->chosenCompressedBufferSize = HARDWARE_BUFFER_SIZE;
        this->offset = HARDWARE_OFFSET;
        this->gain = HARDWARE_GAIN;
    }

    if (QString::compare(codecType, SOFTWARE_CODEC_STRING) == 0)
    {
        this->decodeType = SOFTWARE_DECODE_TYPE;
        this->chosenCompressedBufferSize = SOFTWARE_BUFFER_SIZE;
        this->offset = SOFTWARE_OFFSET;
        this->gain = SOFTWARE_GAIN;
    }

    if (QString::compare(codecType, customCodecString) == 0)
    {
        this->decodeType = this->customAudioDecodeType;
        this->chosenCompressedBufferSize = this->customAudioCompressedBufferSize;
        this->offset = this->customOffset;
        this->gain = this->customGain;
    }
}

void RF4CEAudioHandler::enqueueAudioByte(unsigned char serialByte)
{
    mutex.lock();
    serialDataQueue.append(serialByte);
    mutex.unlock();
}

void RF4CEAudioHandler::setStreamDone()
{
    restartCapture = true;
}

void RF4CEAudioHandler::setRealTimeOutput(bool enable)
{
    realTimeOutput = enable;
}

void RF4CEAudioHandler::run()
{
    int i = 0;
    int count = 0;
    unsigned char serialByte;
    unsigned char* compressed_buffer = NULL;
    int sequence = 0;
    short uncompressed_value;
    int compressedBufferSize = chosenCompressedBufferSize; //int.Parse(msadpcmBufSize.Text);//COMPRESSED_BUFFER_MSADCPM_SIZE;
    int compressedBufferStatePayloadOffset = COMPRESSED_BUFFER_MSADCPM_STATE_PAYLOAD_OFFSET;
    int compressedBufferPayloadSize = compressedBufferSize - COMPRESSED_BUFFER_MSADCPM_STATE_OFFSET; //COMPRESSED_BUFFER_MSADCPM_PAYLOAD_SIZE;
    ImaadpcmState iState;
    HalMsadpcmState mState;
    int8u waveBuffer[] = {0x00, 0x00, 0x00, 0x00};
    int16s sample;
    QString dateTimeString = QDateTime::currentDateTime().toString("MMM-dd_hh-mm-ss");
    QFile uncompressedFile(QString("%1.dat").arg(dateTimeString));
    QDataStream uncompressedOut;

    // Clear any excess data (in case we switch from HW to SW decoding)
    mutex.lock();
    serialDataQueue.clear();
    mutex.unlock();

    // Run until we are shut down
    runStream = true;
    while (runStream)
    {
        // Reset all buffers and sequencing variables
        sequence = 0;
        count = 0;
        waveBuffer[0] = 0x00;
        waveBuffer[1] = 0x00;
        waveBuffer[2] = 0x00;
        waveBuffer[3] = 0x00;

        iState.predictedSample = 0;
        iState.stepIndex = 0;

        mState.predictor = 0;
        mState.delta = 0;
        mState.sample1 = 0;
        mState.sample2 = 0;
        mState.coeff1 = 0;
        mState.coeff2 = 0;

        speakerOutput->resetGlobals();

        // Create new file streams, if saving
        if (saveStreamsValue)
        {
            dateTimeString = QDateTime::currentDateTime().toString("MMM-dd_hh-mm-ss");

            uncompressedFile.setFileName(QString("%1.dat").arg(dateTimeString));
            uncompressedFile.open(QIODevice::WriteOnly);
            uncompressedOut.setDevice(&uncompressedFile);
        }

        if (decodeType == IMADPCM)
        {
            decodeType = IMADPCM;
            compressedBufferSize = chosenCompressedBufferSize; //int.Parse(imadpcmBufSize.Text); //COMPRESSED_BUFFER_IMA_ADCPM_SIZE;
            compressedBufferStatePayloadOffset = COMPRESSED_BUFFER_IMA_ADCPM_STATE_PAYLOAD_OFFSET;
            compressedBufferPayloadSize = compressedBufferSize - COMPRESSED_BUFFER_IMA_ADCPM_STATE_PAYLOAD_OFFSET; //COMPRESSED_BUFFER_IMA_ADCPM_PAYLOAD_SIZE;
        }
        else //if (decodeType == MSADPCM)
        {
            decodeType = MSADPCM;
            compressedBufferSize = chosenCompressedBufferSize; //int.Parse(msadpcmBufSize.Text); //COMPRESSED_BUFFER_MSADCPM_SIZE;
            compressedBufferStatePayloadOffset = COMPRESSED_BUFFER_MSADCPM_STATE_PAYLOAD_OFFSET;
            compressedBufferPayloadSize = compressedBufferSize - COMPRESSED_BUFFER_MSADCPM_STATE_OFFSET; //COMPRESSED_BUFFER_MSADCPM_PAYLOAD_SIZE;
        }
        compressed_buffer = (unsigned char*)malloc(compressedBufferSize);

        // Run until we are shut down or restart the capture
        restartCapture = false;

        while (runStream && !restartCapture)
        {
            mutex.lock();
            int currentSerialDataQueueCount = serialDataQueue.count();
            mutex.unlock();

            // Make sure we have enough data to decompress
            if (currentSerialDataQueueCount >= compressedBufferSize)
            {
                // Fill our compressed_buffer for decompression
                for (i = 0; i < compressedBufferSize; i++)
                {
                    mutex.lock();
                    serialByte = serialDataQueue.at(0);
                    serialDataQueue.remove(0,1);
                    currentSerialDataQueueCount--;
                    mutex.unlock();
                    compressed_buffer[i] = serialByte;
                }

                // microphone driver have a FIFO overflow?
                if ((0x80 & compressed_buffer[COMPRESSED_BUFFER_SEQUENCE_OFFSET]) != 0x00)
                {
                    qDebug() << QString("Microphone Driver reported a FIFO overflow!\r\n");
                }

                // test expected sequence number
                if ((compressed_buffer[COMPRESSED_BUFFER_SEQUENCE_OFFSET] & 0x7f) != (sequence & 0x7f))
                {
                    qDebug() << QString("Sequence mismatch: Expected %1, Received %2").arg(QString::number(sequence & 0x7f), QString::number(compressed_buffer[COMPRESSED_BUFFER_SEQUENCE_OFFSET] & 0x7f));
                    sequence = compressed_buffer[COMPRESSED_BUFFER_SEQUENCE_OFFSET] & 0x7f;
                }

                //pull IMA ADCPM or MSA ADPCM state from buffer
                if (decodeType == IMADPCM)
                {
                    imaadpcmRestoreState(&iState,
                        &compressed_buffer[COMPRESSED_BUFFER_IMA_ADCPM_STATE_OFFSET]);
                }
                else
                {
                    halInternalMsadpcmRestoreState(&mState,
                        &compressed_buffer[COMPRESSED_BUFFER_MSADCPM_STATE_OFFSET]);

                    // save in little endian format
                    waveBuffer[0] = (int8u)(mState.sample2 & 0xff);
                    waveBuffer[1] = (int8u)((mState.sample2 >> 8) & 0xff);

                    waveBuffer[2] = (int8u)(mState.sample1 & 0xff);
                    waveBuffer[3] = (int8u)((mState.sample1 >> 8) & 0xff);

                    sample = mState.sample2;
                    offsetGain(sample, offset, gain, &waveBuffer[0]);
                    if (realTimeOutput)
                    {
                        //speakerOutput->writeData((const char*)&sample, 2);
                        //speakerOutput->writeSample((int16s)((waveBuffer[0] << 8) | waveBuffer[1]));
                        speakerOutput->writeSample(sample);
                    }

                    sample = mState.sample1;
                    offsetGain(sample, offset, gain, &waveBuffer[2]);
                    if (realTimeOutput)
                    {
                        //speakerOutput->writeData((const char*)&sample, 2);
                        //speakerOutput->writeSample((int16s)((waveBuffer[2] << 8) | waveBuffer[3]));
                        speakerOutput->writeSample(sample);
                    }

                    if (saveStreamsValue)
                    {
                        uncompressedOut << (int8u)waveBuffer[0];
                        uncompressedOut << (int8u)waveBuffer[1];
                        uncompressedOut << (int8u)waveBuffer[2];
                        uncompressedOut << (int8u)waveBuffer[3];
                    }
                }
                for (i = compressedBufferStatePayloadOffset; i < compressedBufferSize; i++)
                {
                    // decompress upper nibble
                    if (decodeType == IMADPCM)
                    {
                        uncompressed_value = imaadpcmDecode((int8u)(compressed_buffer[i] >> 4), &iState);
                    }
                    else
                    {
                        uncompressed_value = halInternalMsadpcmDecode((int8u)(compressed_buffer[i] >> 4), &mState);
                    }
                    // save in little endian format
                    waveBuffer[0] = (int8u)(uncompressed_value & 0xff);
                    waveBuffer[1] = (int8u)((uncompressed_value >> 8) & 0xff);

                    sample = uncompressed_value;
                    offsetGain(sample, offset, gain, &waveBuffer[0]);
                    if (realTimeOutput)
                    {
                        //speakerOutput->writeData((const char*)&sample, 2);
                        //speakerOutput->writeSample((int16s)((waveBuffer[0] << 8) | waveBuffer[1]));
                        speakerOutput->writeSample(sample);
                    }

                    // decompress lower nibble
                    if (decodeType == IMADPCM)
                    {
                        uncompressed_value = imaadpcmDecode(compressed_buffer[i], &iState);
                    }
                    else
                    {
                        uncompressed_value = halInternalMsadpcmDecode(compressed_buffer[i], &mState);
                    }
                    // save in little endian format
                    waveBuffer[2] = (int8u)(uncompressed_value & 0xff);
                    waveBuffer[3] = (int8u)((uncompressed_value >> 8) & 0xff);

                    sample = uncompressed_value;
                    offsetGain(sample, offset, gain, &waveBuffer[2]);
                    if (realTimeOutput)
                    {
                        //speakerOutput->writeData((const char*)&sample, 2);
                        //speakerOutput->writeSample((int16s)((waveBuffer[2] << 8) | waveBuffer[3]));
                        speakerOutput->writeSample(sample);
                    }

                    if (saveStreamsValue)
                    {
                        uncompressedOut << (int8u)waveBuffer[0];
                        uncompressedOut << (int8u)waveBuffer[1];
                        uncompressedOut << (int8u)waveBuffer[2];
                        uncompressedOut << (int8u)waveBuffer[3];
                    }
                }
                sequence++;
                count++;
            }
            else
            {
                msleep(1);
            }
        }

        // If we are saving streams write the sequence of bytes to file
        if (saveStreamsValue)
        {
            long bytesCaptured = uncompressedFile.size();
            uncompressedFile.close();

            if (bytesCaptured > 0)
            {
                createWaveFile(dateTimeString);
                qDebug() << QString("Wrote audio stream to %1.wav").arg(dateTimeString);
                // Post to our trace window too
                QMetaObject::invokeMethod((Dialog*)this->parent(),
                                     "postTrace",
                                     Qt::QueuedConnection,
                                     Q_ARG(QString, QString("AUDIO -- %1.wav").arg(dateTimeString)));
                // Post to our trace window too
                QMetaObject::invokeMethod((Dialog*)this->parent(),
                                     "addWaveFile",
                                     Qt::QueuedConnection,
                                     Q_ARG(QString, QString("%1.wav").arg(dateTimeString)));
            }
            else
            {
                QFile::remove(uncompressedFile.fileName());
            }
        }

        qDebug() << QString("Converted %1 buffers (%2 16-bit audio samples) for 16kSps playback!").arg(QString::number(count), QString::number(count * compressedBufferPayloadSize * 2));
    }

    free(compressed_buffer);
}

void RF4CEAudioHandler::createWaveFile(QString filename)
{
    // Setup our wave file for output
    QFile waveFile(QString("%1.wav").arg(filename));
    waveFile.open(QIODevice::WriteOnly);
    QDataStream waveOut(&waveFile);
    waveOut.setByteOrder(QDataStream::LittleEndian);

    // Setup our uncompressed raw data file for input
    QFile uncompressedFile(QString("%1.dat").arg(filename));
    uncompressedFile.open(QIODevice::ReadOnly);
    QDataStream uncompressedOut(&uncompressedFile);

    char rawByte;
    long uncompressedDataByteCount = uncompressedFile.size();
    int sampleCount = (int)uncompressedDataByteCount / waveStreamSampleLength;

    // Create the wave file!
    waveOut.writeRawData("RIFF", 4);
    waveOut << (int)(36 + sampleCount * waveStreamChannels * waveStreamSampleLength);
    waveOut.writeRawData("WAVEfmt ", 8);
    waveOut << (int)16;
    waveOut << (short)1;
    waveOut << (short)waveStreamChannels;
    waveOut << (int)waveStreamSampleRate;
    waveOut << (int)(waveStreamSampleRate * waveStreamSampleLength * waveStreamChannels);
    waveOut << (short)(waveStreamSampleLength * waveStreamChannels);
    waveOut << (short)(8 * waveStreamSampleLength);
    waveOut.writeRawData("data", 4);
    waveOut << (int)(sampleCount * waveStreamSampleLength);
    QByteArray rawBytes(uncompressedFile.readAll());
    for (int i = 0; i < uncompressedDataByteCount; i++)
    {
        rawByte = rawBytes.at(i);
        waveOut.writeRawData(&rawByte, 1);//readBytes;
    }

    // Close our files
    waveFile.close();
    uncompressedFile.close();

    // Remove the raw data file, not needed anymore
    QFile::remove(uncompressedFile.fileName());
}


// Global Audio Functions

void imaadpcmRestoreState(ImaadpcmState *stateDst, const int8u *stateSrc)
{
    // Restore state index and predicated sample from a packed form
    stateDst->stepIndex = stateSrc[0];
    stateDst->predictedSample = (int16s)((((int16u)stateSrc[2]) << 8)
                                        + ((int16u)stateSrc[1]      ));
}

int16s imaadpcmDecode(int8u compressedCode, ImaadpcmState *imaadpcmState)
{
// Promote the current step size and predicted sample to int32s
int32s currentStep = (int32s)imaadpcmStepTable[imaadpcmState->stepIndex];
int32s predictedSample = (int32s)imaadpcmState->predictedSample;
int32s diff;

    // Calculate the difference
    diff = currentStep >> 3;
    if (compressedCode & 1) {
      diff += currentStep >> 2;
    }
    if (compressedCode & 2) {
      diff += currentStep >> 1;
    }
    if (compressedCode & 4) {
      diff += currentStep;
    }

    // Calc the new predicted sample with saturation logic from int32s->int16s
    predictedSample += (compressedCode & 8) ? -diff : diff;
    if (predictedSample > 32767) {
      predictedSample = 32767;
    } else if (predictedSample < -32768) {
      predictedSample = -32768;
    }
    imaadpcmState->predictedSample = (int16s)predictedSample;

    // Update the step index for the next sample
    imaadpcmState->stepIndex += imaadpcmIndexAdjust[compressedCode & 0x0f];
    if (imaadpcmState->stepIndex < 0) {
      imaadpcmState->stepIndex = 0;
    } else if (imaadpcmState->stepIndex > 88) {
      imaadpcmState->stepIndex = 88;
    }

    return imaadpcmState->predictedSample;
}

int16s halInternalMsadpcmDecode(int8u compressedCode, HalMsadpcmState *state)
{
  int32s predictedSample, uncompressedCode;

  compressedCode &= 0x0F;   // Ensure upper compressedCode is 0

  uncompressedCode = (int32s)compressedCode; // Convert code to int32s
  if (compressedCode & 0x08) {
    uncompressedCode -= 0x10;
  }
  // Calculate sample
  predictedSample = ((int32s)state->sample1 * state->coeff1
                     + (int32s)state->sample2 * state->coeff2) >> 8;
  predictedSample += uncompressedCode * state->delta;

  // Saturate to int16s
  if (predictedSample > 32767) {
    predictedSample = 32767;
  } else if (predictedSample < -32768) {
      predictedSample = -32768;
  }

  // Update state variables
  state->sample2 = state->sample1;
  state->sample1 = (int16s)predictedSample;

  state->delta = (msadpcmAdaptTable[compressedCode] * state->delta) >> 8;
  if (state->delta < 16) {
    state->delta = 16;
  }

  return state->sample1;
}

void halInternalMsadpcmRestoreState(HalMsadpcmState *stateDst, const int8u *stateSrc)
{
  // Restore state from a packed form
  stateDst->predictor = stateSrc[0];
  if (stateSrc[0] > 6) {
    stateDst->predictor = 6; // predictor range 0-6
  }

  stateDst->delta  = (int16s)((((int16u)stateSrc[2]) << 8)
                                + ((int16u)stateSrc[1]));

  stateDst->sample1 = (int16s)((((int16u)stateSrc[4]) << 8)
                                + ((int16u)stateSrc[3]));
  stateDst->sample2 = (int16s)((((int16u)stateSrc[6]) << 8)
                                + ((int16u)stateSrc[5]));

  stateDst->coeff1 = msadpcmAdaptCoeff1[stateDst->predictor];
  stateDst->coeff2 = msadpcmAdaptCoeff2[stateDst->predictor];
}

void offsetGain(int16s RawSample, int16s Offset, int16s Gain, int8u *Buffer)
{
    int32s Sample;

    // swap from big endian to little endian
    //RawSample = ((RawSample << 8) & 0xff00) | ((RawSample >> 8) & 0x00ff);
    // promote from int16s to int32s
    Sample = (int32s)RawSample;

    // Apply offset and gain
    Sample -= (int32s)Offset;
    Sample *= (int32s)Gain;

    // clamp sample to 16-bit value
    if (Sample > 32767) Sample = 32767;
    else if (Sample < -32768) Sample = -32768;

    // save to byte array buffer in little endian format
    Buffer[0] = (int8u)(Sample & 0xff);
    Buffer[1] = (int8u)((Sample >> 8) & 0xff);
}
