#ifndef RF4CEAUDIOHANDLER_H
#define RF4CEAUDIOHANDLER_H

#include <QDebug>
#include <QObject>
#include <QThread>
#include <QIODevice>
#include <QByteArray>
#include <QtMultimedia/QAudioOutput>
#include <QMutex>
#include <QFile>
#include <QDateTime>
#include "audioout48.h"

typedef struct ImaadpcmState {
   int8s stepIndex;           // Index into imaadpcmStepTable
   int16s predictedSample;    // Last predicted sample
} ImaadpcmState;

#define IMA_ADCPM_STATE_SIZE (sizeof(int8s) + sizeof(int16s)) // PACKED SIZE

typedef struct HalMsadpcmState {
  int8u predictor;            // Index into msadpcmAdaptCoeffx tables
  int16u delta;               // Quantization scale
  int16s sample1;             // Second sample of block (full PCM format)
  int16s sample2;             // First sample of block (full PCM format)

                              // Temp variables
  int32s coeff1;              // Promoted msadpcmAdaptCoeff1[State->predictor]
  int32s coeff2;              // Promoted msadpcmAdaptCoeff2[State->predictor]
} HalMsadpcmState;

#define MSADCPM_STATE_SIZE (sizeof(int8u)+sizeof(int16u)+2*sizeof(int16s)) // PACKED SIZE
#define MSADCPM_SAMPLES_IN_HEADER 2;

enum DecodeType
{
    IMADPCM,
    MSADPCM
};

static const int8s imaadpcmIndexAdjust[16] = {
   -1, -1, -1, -1, 2, 4, 6, 8, // +0 - +3 => decrease, +4 - +7 => increase step
   -1, -1, -1, -1, 2, 4, 6, 8  // -0 - -3 => decrease, -4 - -7 => increase step
};

static const int16s imaadpcmStepTable[89] = {
   7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 21, 23, 25, 28, 31, 34, 37, 41, 45,
   50, 55, 60, 66, 73, 80, 88, 97, 107, 118, 130, 143, 157, 173, 190, 209, 230,
   253, 279, 307, 337, 371, 408, 449, 494, 544, 598, 658, 724, 796, 876, 963,
   1060, 1166, 1282, 1411, 1552, 1707, 1878, 2066, 2272, 2499, 2749, 3024,
   3327, 3660, 4026, 4428, 4871, 5358, 5894, 6484, 7132, 7845, 8630, 9493,
   10442, 11487, 12635, 13899, 15289, 16818, 18500, 20350, 22385, 24623, 27086,
   29794, 32767
};

static const int16s msadpcmAdaptTable[] = {
    230, 230, 230, 230, 307, 409, 512, 614,
    768, 614, 512, 409, 307, 230, 230, 230
};

static const int16s msadpcmAdaptCoeff1[] = {
    256, 512, 0, 192, 240, 460, 392
};

static const int16s msadpcmAdaptCoeff2[] = {
    0, -256, 0, 64, 0, -208, -232
};

const int COMPRESSED_BUFFER_SEQUENCE_OFFSET = 0;

const int COMPRESSED_BUFFER_IMA_ADCPM_STATE_OFFSET = (COMPRESSED_BUFFER_SEQUENCE_OFFSET + 1);
const int COMPRESSED_BUFFER_IMA_ADCPM_STATE_PAYLOAD_OFFSET = (COMPRESSED_BUFFER_IMA_ADCPM_STATE_OFFSET + IMA_ADCPM_STATE_SIZE);
//const int COMPRESSED_BUFFER_IMA_ADCPM_SIZE = 80; // Changed to be variable based on input
//const int COMPRESSED_BUFFER_IMA_ADCPM_PAYLOAD_SIZE = (COMPRESSED_BUFFER_IMA_ADCPM_SIZE - COMPRESSED_BUFFER_IMA_ADCPM_STATE_PAYLOAD_OFFSET);

const int COMPRESSED_BUFFER_MSADCPM_STATE_OFFSET = (COMPRESSED_BUFFER_SEQUENCE_OFFSET + 1);
const int COMPRESSED_BUFFER_MSADCPM_STATE_PAYLOAD_OFFSET = (COMPRESSED_BUFFER_MSADCPM_STATE_OFFSET + MSADCPM_STATE_SIZE);
//const int COMPRESSED_BUFFER_MSADCPM_SIZE = 79; // Changed to be variable based on input
//const int COMPRESSED_BUFFER_MSADCPM_PAYLOAD_SIZE = (COMPRESSED_BUFFER_MSADCPM_SIZE - COMPRESSED_BUFFER_MSADCPM_STATE_OFFSET);

// Wave Constants
#define WAVE_CHANNELS 1
#define WAVE_SAMPLE_RATE 16000
#define WAVE_SAMPLE_SIZE 2

// Gain adjust Constants
#define INMP421_TYP_OFFSET 1638 // 5% of FS => round(32768*0.05) = 1638
#define TLV320DAC3203_GAIN 4 // driver sets TI codec to 12dB gain

#define DEFAULT_OFFSET (INMP421_TYP_OFFSET * TLV320DAC3203_GAIN)
#define DEFAULT_GAIN 6 // apply 12dB additional gain

#define HARDWARE_CODEC_STRING "Hardware Codec Settings"
#define HARDWARE_DECODE_TYPE MSADPCM
#define HARDWARE_BUFFER_SIZE 103
#define HARDWARE_OFFSET 0
#define HARDWARE_GAIN 1

#define SOFTWARE_CODEC_STRING "Software Codec Settings"
#define SOFTWARE_DECODE_TYPE IMADPCM
#define SOFTWARE_BUFFER_SIZE 104
#define SOFTWARE_OFFSET 0
#define SOFTWARE_GAIN 1

// Global Audio Functions
int16s imaadpcmDecode(int8u compressedCode, ImaadpcmState *imaadpcmState);
void imaadpcmRestoreState(ImaadpcmState *stateDst, const int8u *stateSrc);
int16s halInternalMsadpcmDecode(int8u compressedCode,
                                HalMsadpcmState *state);
void halInternalMsadpcmRestoreState(HalMsadpcmState *stateDst,
                                    const int8u *stateSrc);

class RF4CEAudioHandler : public QThread
{
    Q_OBJECT

public:
    RF4CEAudioHandler(QObject *parent = 0, QString codecType = HARDWARE_CODEC_STRING);
    ~RF4CEAudioHandler();

    void setCodecValues(QString codecType);
    void setCustomCodecSettings(QString codecString,
                                DecodeType decodeType,
                                int bufferSize,
                                int16s offset,
                                int16s gain);
    bool startStream();
    void stopStream();
    void enqueueAudioByte(unsigned char serialByte);
    void setStreamDone();
    void setRealTimeOutput(bool enable);

protected:
    void run();

private:
    void createWaveFile(QString filename);

    int waveStreamChannels;
    int waveStreamSampleRate;
    int waveStreamSampleLength;

    volatile bool runStream;
    volatile bool realTimeOutput;
    volatile bool saveStreamsValue;
    volatile bool restartCapture;
    DecodeType decodeType;
    int chosenCompressedBufferSize;
    int16s offset;
    int16s gain;

    QMutex mutex;
    QAudioDeviceInfo audioDevice;
    QAudioOutput* audioOutput;
    //QIODevice* output; // not owned
    AudioOut48* speakerOutput;
    QAudioFormat audioFormat;
    QByteArray serialDataQueue;

    // Custom audio settings
    QString customCodecString;
    DecodeType customAudioDecodeType;
    int customAudioCompressedBufferSize;
    int16s customOffset;
    int16s customGain;
};

#endif // RF4CEAUDIOHANDLER_H
