#include "rf4ceprotocolhandler.h"
#include "dialog.h"

RF4CEProtocolHandler::RF4CEProtocolHandler(QObject *parent, RF4CEAudioHandler* pAudioHandler) : QThread(parent)
{
    this->setParent(parent);
    this->audioHandler = pAudioHandler;
}

RF4CEProtocolHandler::~RF4CEProtocolHandler()
{

}

void RF4CEProtocolHandler::startStream()
{
    this->start();
}

void RF4CEProtocolHandler::stopStream()
{
    // Shut down the thread and wait for completion
    runStream = false;
    // Add timeout to robustify this
    while(this->isRunning());
}

void RF4CEProtocolHandler::enqueuePacket(QList<char>* packet)
{
    mutex.lock();
    this->packetQueue.enqueue(packet);
    mutex.unlock();
}

void RF4CEProtocolHandler::sendIdentifyPacket(SerialHandler* serialHandler)
{
    unsigned char bufferSize = 2;
    unsigned char buffer[2];
    unsigned char uSec = 8; // Time to alert
    unsigned char uFlg = 0;
    uFlg |= 0x01; //uIDENTIFY_FLAG_STOP_ON_ACTION
    uFlg |= 0x02; //uIDENTIFY_FLAG_FLASH_LIGHT
    uFlg |= 0x04; //uIDENTIFY_FLAG_MAKE_SOUND
    uFlg |= 0x08; //uIDENTIFY_FLAG_VIBRATE

    buffer[0] = uFlg;
    buffer[1] = uSec;
    sendToTarget(serialHandler, IdentifyAck, buffer, bufferSize);
}

void RF4CEProtocolHandler::sendPairingPacket(SerialHandler* serialHandler)
{
    unsigned char bufferSize = 0;
    unsigned char buffer[1];

    sendToTarget(serialHandler, BindAck, buffer, bufferSize);
}

void RF4CEProtocolHandler::sendToTarget(SerialHandler* serialHandler, unsigned char uMid, unsigned char *buffer, unsigned char bufferSize)
{
    int iAppTxBufLen;
    int iPreTxBufLen;

    iAppTxBufLen = appEncode(uMid, buffer, bufferSize);
    iPreTxBufLen = preEncode(iAppTxBufLen);
    serialHandler->write(xPreTxBufDat, iPreTxBufLen);
}

int RF4CEProtocolHandler::appEncode(unsigned char uMid, unsigned char *buffer, unsigned char bufferSize)
{
    int iAppTxBufLen = 0;

    xAppTxBufDat[iAppTxBufLen++] = MESSAGE_HEADER_VERSION_INDEX;
    xAppTxBufDat[iAppTxBufLen++] = uMid;
    xAppTxBufDat[iAppTxBufLen++] = bufferSize;
    memcpy(xAppTxBufDat+iAppTxBufLen, buffer, bufferSize);
    iAppTxBufLen += bufferSize;

    return iAppTxBufLen;
}

int RF4CEProtocolHandler::preEncode(int iAppTxBufLen)
{
    int iPreTxBufLen = 0;
    unsigned char uSum = 0;

    xPreTxBufDat[iPreTxBufLen++] = MESSAGE_FRAME_START;
    for (int i = 0; i < iAppTxBufLen; i++)
    {
        uSum ^= xAppTxBufDat[i];
        preEsc(&iPreTxBufLen, xAppTxBufDat[i]);
    }
    preEsc(&iPreTxBufLen, uSum);
    xPreTxBufDat[iPreTxBufLen++] = MESSAGE_FRAME_END;

    return iPreTxBufLen;
}

void RF4CEProtocolHandler::preEsc(int *iPreTxBufLen, unsigned char uDat)
{
    if ((uDat == MESSAGE_FRAME_START) || (uDat == MESSAGE_FRAME_END) || (uDat == MESSAGE_ESCAPE_CHARACTER))
    {
        xPreTxBufDat[(*iPreTxBufLen)++] = MESSAGE_FRAME_START;
        xPreTxBufDat[(*iPreTxBufLen)++] = (unsigned char)(uDat ^ MESSAGE_ESCAPE_CHARACTER_MASK);
    }
    else
    {
        xPreTxBufDat[(*iPreTxBufLen)++] = uDat;
    }
}

void RF4CEProtocolHandler::run()
{
    QString debugString;
    int checkSum = 0x00;
    QList<char> *currentPacket = NULL;

    // Run until we are shut down
    runStream = true;
    while (runStream)
    {
        checkSum = 0x00;

        // Grab the next packet and process it
        mutex.lock();
        currentPacket = NULL;
        if (packetQueue.count())
        {
            currentPacket = packetQueue.dequeue();
        }
        mutex.unlock();

        if (currentPacket != NULL)
        {
            for (int i = 0; i < currentPacket->count(); i++)
            {
                // If we got an escape character then we will convert the next byte
                if (currentPacket->at(i) == MESSAGE_ESCAPE_CHARACTER)
                {
                    currentPacket->replace(i+1, currentPacket->at(i+1) ^ MESSAGE_ESCAPE_CHARACTER_MASK);
                    currentPacket->removeAt(i);
                }

                checkSum = checkSum ^ currentPacket->at(i);
            }

            if (currentPacket->at(MESSAGE_HEADER_VERSION_INDEX) != MESSAGE_VERSION_CURRENT)
            {
                qDebug() << QString("Error in packet, unknown version. Version=0x%1, Expected=0x%2").arg(
                                QString::number((int)currentPacket->at(MESSAGE_HEADER_VERSION_INDEX), 16),
                                QString::number((int)MESSAGE_VERSION_CURRENT, 16));
                continue;
            }

            if (checkSum != 0x00)
            {
                qDebug() << QString("Error in packet, checksum != 0x00. checksum=%1").arg(
                                QString::number((int)checkSum, 16));
                continue;
            }

            if (currentPacket->at(MESSAGE_HEADER_LENGTH_INDEX) != currentPacket->count() - MESSAGE_HEADER_SIZE - 1)
            {
                qDebug() << QString("Error in packet, header length (%1) != actual length (%2)").arg(
                                QString::number((int)currentPacket->at(MESSAGE_HEADER_LENGTH_INDEX)),
                                QString::number(currentPacket->count() - MESSAGE_HEADER_SIZE));
                continue;
            }

            // If the packet is audio, pass it off to the audio object
            if ((MessageType)(currentPacket->at(MESSAGE_HEADER_TYPE_INDEX)) == Audio)
            {
                //qDebug() << QString("PACKET -- Ver: %1, Type: %2, Len: %3: *AUDIO*").arg(
                //                QString::number((int)currentPacket->at(MESSAGE_HEADER_VERSION_INDEX), 16),
                //                QString::number((int)currentPacket->at(MESSAGE_HEADER_TYPE_INDEX)),
                //                QString::number((int)currentPacket->at(MESSAGE_HEADER_LENGTH_INDEX)));

                // Write the payload offset by the header size, reduced by 1 for the checksum
                for (int i = MESSAGE_HEADER_SIZE; i < (currentPacket->count() - 1); i++)
                {
                    audioHandler->enqueueAudioByte(currentPacket->at(i));
                }
                continue;
            }

            // If we now have a packet, see what it is - the next three bytes
            // [version] [messageType] [messageLength]

            debugString = QString("PACKET -- Ver: 0x%1, Type: 0x%2, Len: %3: ").arg(
                            QString::number((int)currentPacket->at(MESSAGE_HEADER_VERSION_INDEX), 16),
                            QString::number((int)currentPacket->at(MESSAGE_HEADER_TYPE_INDEX), 16),
                            QString::number((int)currentPacket->at(MESSAGE_HEADER_LENGTH_INDEX)));

            // Write the payload offset by the header size, reduced by 1 for the checksum
            for (int i = MESSAGE_HEADER_SIZE; i < (currentPacket->count() - 1); i++)
            {
                switch ((MessageType)(currentPacket->at(MESSAGE_HEADER_TYPE_INDEX)))
                {
                    case Action:
                        if (i == (MESSAGE_HEADER_SIZE + MESSAGE_KEYCODE_PAYLOAD_OFFSET))
                        {
                            // This is spoofing the audio key
                            if (currentPacket->at(i-1) != 32)
                            {
                                QMetaObject::invokeMethod((Dialog*)this->parent(),
                                                     "keyCodePressed",
                                                     Qt::QueuedConnection,
                                                     Q_ARG(int, currentPacket->at(i)),
                                                     Q_ARG(unsigned char, currentPacket->at(i-3)));
                            }
                            else
                            {
                                if (currentPacket->at(i) == 114)
                                {
                                    QMetaObject::invokeMethod((Dialog*)this->parent(),
                                                         "keyCodePressed",
                                                         Qt::QueuedConnection,
                                                         Q_ARG(int, -1),
                                                         Q_ARG(unsigned char, 0x03));
                                }
                                else
                                {
                                    QMetaObject::invokeMethod((Dialog*)this->parent(),
                                                         "keyCodePressed",
                                                         Qt::QueuedConnection,
                                                         Q_ARG(int, -2),
                                                         Q_ARG(unsigned char, 0x03));
                                }
                            }
                        }
                    case GetStatusReq:
                    case GetStatusAck:
                        debugString += QString("0x%1, ").arg(QString::number(currentPacket->at(i), 16)); // Print bytes directly to console
                        break;
                    default:
                        debugString += QString("0x%1, ").arg(QString::number(currentPacket->at(i), 16)); // Unknown message type
                        break;
                }
            }

            debugString += "END";
            qDebug() << debugString;

            // Post to our trace window too
            QMetaObject::invokeMethod((Dialog*)this->parent(),
                                 "postTrace",
                                 Qt::QueuedConnection,
                                 Q_ARG(QString, debugString));

            delete currentPacket;
        }
        else
        {
            msleep(1);
        }
    }
}
