/**
 * @file        ueic_defs.h
 * @brief       common definitions for quickset libary
 *
 */

/*
 * Copyright 2010 Universal Electronics Inc. All Rights
 * Reserved.
 *
 * NOTICE:  You are permitted to use this file only in
 * accordance with the terms of the Universal Electronic Inc.
 * license agreement under which it was obtained.
 * Redistribution, publication, or distribution in any form is
 * not permitted without a separate license.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT OWNER(S) BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 */


#ifndef _UEIC_DEFS_H
#define _UEIC_DEFS_H

#include "ueic_status.h"

/* typedef    void  *  	QS_IR_list_t;    ;<deprecated rev 2.4.8> */ 
typedef const char *        QS_name_t  ;
typedef       char *        QS_str_t   ;
typedef signed int          QS_index_t ;
typedef signed int          QS_size_t  ;

#if HAVE_STDINT_H == 1

#include <stdint.h>
typedef uint32_t           QS_u32      ;
typedef uint16_t           QS_u16      ;
typedef uint8_t            QS_u8       ;

typedef int32_t            QS_s32      ;
typedef int16_t            QS_s16      ;
typedef int8_t             QS_s8       ;

#else

#ifdef USE_8_BYTE_LONG
typedef unsigned int        QS_u32     ;
#else
typedef unsigned long       QS_u32     ;
#endif
typedef unsigned short      QS_u16     ;
typedef unsigned char       QS_u8      ;

/* typedef long long    		QS_s64     ;<deprecated rev 2.4.8> */
typedef long         		QS_s32     ;
typedef short        		QS_s16     ;
typedef char         		QS_s8      ;

#endif 

#define QS_Null 0

#define NUM_ELEMENTS(a) (sizeof(a) / sizeof(a[0]))


#ifndef kNull
   #define kNull 0
#endif
#ifndef NULL
   #define NULL 0
#endif

#ifndef HIGH
   #define HIGH 1
#endif

#ifndef LOW
   #define LOW 0
#endif

typedef unsigned int		QS_uint;
typedef signed int			QS_sint;

/* This really belongs in limits.h but we don't have such a thing and there's not much to it. */

#define S24_MAX	8388607
#define S24_MIN	(-S24_MAX - 1)
#define U24_MAX (S24_MAX * 2 + 1)


typedef enum{
    QS_fail = -1,
    QS_ok = 0
} QS_status_t;


typedef enum{
    QS_true = (1==1),
    QS_false = (1!=1)
} QS_bool_t;

typedef enum{

    QS_st_invalid = -1,
    QS_st_init,
    QS_st_has_type,
    QS_st_has_brand,
    QS_st_has_model,
    QS_st_has_codeset

} QS_state_t;


/**
 * @brief      Collection of fuzzy logic flags
 * @ingroup	   fuzzy_flags
 * @since	   1.2
 *
 * This collection of fuzzy flag bits can store fuzzy flags using bit-wise-or.
 * E.g. the following code stores the FUZZY_2ND_SORT_ASC and 
 * FUZZY_CASE_SENSITIVE flags in a single fuzzy_flags_t 
 * variable. 
 * @code
 *     fuzzy_flags_t flags = 0;
 *     flags = FUZZY_2ND_SORT_ASC | FUZZY_CASE_SENSITIVE;
 * @endcode
 */

typedef enum ueic_fuzzyFlags_t
{
/**
 * @brief      Used if no flags are needed
 * @ingroup	   fuzzy_flags
 * @since	   2.4.5
 *
 * This flag can be passed in to represent no flag options.  If other
 * flags are passed in with this flag, those other flags will take
 * priority over this flag.
 */
    FUZZY_CLEAR = 0x0000,

/**
 * @brief      Flags fuzzy logic to standardize ambiguous characters
 * @ingroup	   fuzzy_flags
 * @since	   1.2
 *
 * If present, ambiguous characters (E.g. l and 1) are taken as the same
 * character when compared.
 */
    FUZZY_STANDARDIZE_AMBIGUOUS_CHARS = 0x0001,

/**
 * @brief      Sort models of the same fuzzy value in ascending alphanumeric
 *             order
 * @ingroup	   fuzzy_flags
 * @since	   1.2
 *
 * If present, all returned models with the same fuzzy value will be sorted
 * in ascending alphanumeric order.  This flag and the FUZZY_2ND_SORT_DESC
 * flag are mutually exclusive and functions passed in both flags should
 * return a ueic_mutual_exclusivity_error.  If neither a FUZZY_2ND_SORT_ASC or
 * FUZZY_2ND_SORT_DESC flag is present, returned models with the same fuzzy
 * value will not be sorted.
 */
    FUZZY_2ND_SORT_ASC = 0x0020,

/**
 * @brief      Sort models of the same fuzzy value in descending alphanumeric
 *             order
 * @ingroup	   fuzzy_flags
 * @since	   1.2
 *
 * If present, all returned models with the same fuzzy value will be sorted
 * in descending alphanumeric order.  This flag and the FUZZY_2ND_SORT_ASC
 * flag are mutually exclusive and functions passed in both flags should
 * return a ueic_mutual_exclusivity_error.  If neither a FUZZY_2ND_SORT_ASC or
 * FUZZY_2ND_SORT_DESC flag is present, returned models with the same fuzzy
 * value will not be sorted.
 */
    FUZZY_2ND_SORT_DESC = 0x0040,

/**
 * @brief      Fuzzy string comparisons are case sensitive
 * @ingroup	   fuzzy_flags
 * @since	   1.2
 *../src/localdb.c:44:1:
 * If present, all fuzzy string comparison will be case sensitive.  If
 * this flag is not present, then fuzzy string comparison will NOT be case
 * sensitive.
 */
    FUZZY_CASE_SENSITIVE	= 0x0080,

/**
 * @brief      Non Critical Chararcters will be removed from the string before
 *              matching the string.
 * @ingroup    fuzzy_flags
 * @since      1.2
 *
 * If present, Non Critical characters (E.g. '.','-', etc) are taken as removable
 * character from the String.
 */
    FUZZY_NONCRITICAL_CHARS = 0x0100,

/**
 * @brief      Marching at leading position scores higher.
 * @since      1.4.5
 *
 * If present, strings with matching leading charactors scores
 *              higher than others .
 */
    FUZZY_FAVOR_LEADING_CHAR    = 0x0200,

/**
 * @brief      Default value of fuzzy properties 
 * @ingroup    fuzzy_flags 
 * @since      1.2 
 *  
 * If fuzzy property is not set, by default model strings with 
 * matching leading characters score higher and all returned 
 * models with the same fuzzy value will be sorted in ascending 
 * alphanumeric order. 
 */
   FUZZY_DEFAULT = FUZZY_FAVOR_LEADING_CHAR | FUZZY_2ND_SORT_ASC

} ueic_fuzzyFlags_t;

/**
 * @brief       Defines modes encountered in a remote control
 *              unit
 */
typedef enum remote_mode_e
{
	unlocked = 0,
	cable_mode,
	tv_mode,
	dvd_mode,
	aud_mode,
	aux_mode,
	sat_mode,
	aux2_mode,
	aux3_mode,
	INVALID_mode
}remote_mode_t;

/* Optimized Setup Maps */

/**
 * @brief       Defines the key codes.
 */
typedef enum
{
    kKey_INVALID                 = -1,

    kKey_POWER                   =  1,
    kKey_POWER_ON                =  2,
    kKey_POWER_OFF               =  3,
    kKey_CHANNEL_UP              =  4,
    kKey_CHANNEL_DOWN            =  5,
    kKey_VOLUME_UP               =  6,
    kKey_VOLUME_DOWN             =  7,
    kKey_MUTE                    =  8,
    kKey_DIGIT_1                 =  9,
    kKey_DIGIT_2                 =  10,
    kKey_DIGIT_3                 =  11,
    kKey_DIGIT_4                 =  12,
    kKey_DIGIT_5                 =  13,
    kKey_DIGIT_6                 =  14,
    kKey_DIGIT_7                 =  15,
    kKey_DIGIT_8                 =  16,
    kKey_DIGIT_9                 =  17,
    kKey_DIGIT_0                 =  18,
    kKey_CHANNEL_ENTER           =  19,
    kKey__100                    =  20,
    kKey_LAST_CHANNEL            =  21,
    kKey_INPUT                   =  22,
    kKey_EXTERNAL                =  23,
    kKey_PLAY                    =  24,
    kKey_STOP                    =  25,
    kKey_PAUSE                   =  26,
    kKey_REWIND                  =  27,
    kKey_FAST_FORWARD            =  28,
    kKey_RECORD                  =  29,
    kKey_SKIP_FWD                =  30,
    kKey_SKIP_REVERSE            =  31,
    kKey_LIVE                    =  32,
    kKey_SOURCE_MENU             =  33,
    kKey_DEVICE_MENU             =  34,
    kKey_GUIDE                   =  35,
    kKey_EXIT                    =  36,
    kKey_BACK                    =  37,
    kKey_CURSOR_UP               =  38,
    kKey_CURSOR_DOWN             =  39,
    kKey_CURSOR_LEFT             =  40,
    kKey_CURSOR_RIGHT            =  41,
    kKey_MENU_SELECT             =  42,
    kKey_PAGE_UP                 =  43,
    kKey_PAGE_DOWN               =  44,
    kKey_FAVORITE                =  45,
    kKey_DISPLAY                 =  46,
    kKey_FORMAT                  =  47,
    kKey_SAP                     =  48,     
    kKey_SURROUND                =  49,     
    kKey_SLOW                    =  50,      
    kKey_DELIMITER               =  51,      
    kKey_RANDOM                  =  52,      
    kKey_PIP_ON                  =  53,      
    kKey_PIP_OFF                 =  54,      
    kKey_PIP_FREEZE              =  55,      
    kKey_PIP_SWAP                =  56,      
    kKey_PIP_MOVE                =  57,      
    kKey_PIP_INPUT               =  58,      
    kKey_PIP_CHANNEL_UP          =  59,      
    kKey_PIP_CHANNEL_DOWN        =  60,      
    kKey_PIP_MULTI               =  61,   
    kKey_INPUT_1                 =  62,
    kKey_INPUT_2                 =  63,
    kKey_INPUT_3                 =  64,
    kKey_INPUT_4                 =  65,
    kKey_INPUT_5                 =  66,
    kKey_INPUT_6                 =  67,
    kKey_INPUT_7                 =  68,
    kKey_INPUT_8                 =  69,
    kKey_INPUT_9                 =  70,
    kKey_INPUT_10                =  71,
    kKey_List                    =  72,
    kKey_HDMI1                   =  73,
    kKey_HDMI2                   =  74,
    kKey_HDMI3                   =  75,
    kKey_HDMI4                   =  76,
    kKey_RED                     =  77,
    kKey_GREEN                   =  78, 
    kKey_YELLOW                  =  79, 
    kKey_BLUE                    =  80, 
    kKey_HDMI5                   =  81,
    kKey_A                       =  82,
    kKey_B                       =  83,
    kKey_C                       =  84,
    kKey_D                       =  85,
    kKey_WEB_INTERNET            =  86,
    kKey_WIDGETS_APPS            =  87,
    kKey_LINK                    =  88,
    kKey_3D_VIDEO                =  89,
    kKey_PICTURE_MODE            =  90,
    kKey_SOUND_MODE              =  91,
    kKey_CLEAR_CANCEL            =  92,
    kKey_STANDBY                 =  93,
    kKey_SLEEP                   =  94,
    kKey_OPEN_CLOSE_EJECT        =  95,
    kKey_PVR_MENU                =  96,
    kKey_POPUP_MENU              =  97,
    kKey_TITLE_MENU              =  98,
    kKey_QUICK_SKIP              =  99,
    kKey_INSTANT_REPLAY          =  100,
    kKey_DAY__                   =  101,
    kKey_DAY_                    =  102,
    kKey_SUBTITLE                =  103,
    kKey_RECORD_SPEED            =  104,
    kKey_RECORDINGS              =  105,
    kKey_TEXT_ON                 =  106,
    kKey_TEXT_OFF                =  107,
    kKey_TEXT_HOLD               =  108,
    kKey_TEXT_MIX                =  109, 
    kKey_TEXT_EXPAND             =  110, 
    kKey_TIME                    =  111, 
    kKey_TEXT_UPDATE             =  112, 
    kKey_TEXT_I_MARK             =  113, 
    kKey_TEXT_REVEAL             =  114, 
    kKey_WHITE                   =  115, 
    kKey_PURPLE                  =  116, 
    kKey_TV_DTV                  =  117, 
    kKey_DTV                     =  118, 
    kKey_ATV                     =  119, 
    kKey_DASH                    =  120, 
    kKey_HOME                    =  121, 
    kKey_INTERACTIVE             =  122, 
    kKey_SERVICES                =  123, 
    kKey_BLU_RAY                 =  124, 
    kKey_PC                      =  125, 
    kKey_COMPONENT_1             =  126, 
    kKey_COMPONENT_2             =  127, 
    kKey_COMPONENT_3             =  128, 
    kKey_DVI_1                   =  129, 
    kKey_DVI_2                   =  130, 
    kKey_GAME                    =  131, 
    kKey_AUX_2                   =  132, 
    kKey_AUX_3                   =  133, 
    kKey_Netflix                 =  134, 
    kKey_Vudu                    =  135, 
    kKey_HELP                    =  136,
    kKey_RECORD_STOP             =  137, /* RESERVED GLOBAL KEYS 138 TO 175 */
    kKey_SD                      =  176, /* Pick for Japan Market Codesets ONLY  176 TO 244 */
    kKey_DIGIT_11                =  177, 
    kKey_DIGIT_12                =  178, 
    kKey_DATA                    =  179, 
    kKey_ANALOG_TERREST          =  180, 
    kKey_DIGITAL_TERREST         =  181, 
    kKey_BROADCAST_SAT           =  182, 
    kKey_COM_SAT                 =  183, 
    kKey_TUNER_SELECT            =  184, 
    kKey_Analog_1                =  185, 
    kKey_Analog_2                =  186, 
    kKey_Analog_3                =  187, 
    kKey_Analog_4                =  188, 
    kKey_Analog_5                =  189, 
    kKey_Analog_6                =  190, 
    kKey_Analog_7                =  191, 
    kKey_Analog_8                =  192, 
    kKey_Analog_9                =  193, 
    kKey_Analog_10               =  194, 
    kKey_Analog_11               =  195, 
    kKey_Analog_12               =  196, 
    kKey_Digital_1               =  197, 
    kKey_Digital_2               =  198, 
    kKey_Digital_3               =  199, 
    kKey_Digital_4               =  200, 
    kKey_Digital_5               =  201, 
    kKey_Digital_6               =  202, 
    kKey_Digital_7               =  203, 
    kKey_Digital_8               =  204, 
    kKey_Digital_9               =  205, 
    kKey_Digital_10              =  206, 
    kKey_Digital_11              =  207, 
    kKey_Digital_12              =  208, 
    kKey_Broadcast_Sat_1         =  209, 
    kKey_Broadcast_Sat_2         =  210, 
    kKey_Broadcast_Sat_3         =  211, 
    kKey_Broadcast_Sat_4         =  212, 
    kKey_Broadcast_Sat_5         =  213, 
    kKey_Broadcast_Sat_6         =  214, 
    kKey_Broadcast_Sat_7         =  215, 
    kKey_Broadcast_Sat_8         =  216, 
    kKey_Broadcast_Sat_9         =  217, 
    kKey_Broadcast_Sat_10        =  218, 
    kKey_Broadcast_Sat_11        =  219, 
    kKey_Broadcast_Sat_12        =  220, 
    kKey_Comm_Sat_1              =  221, 
    kKey_Comm_Sat_2              =  222, 
    kKey_Comm_Sat_3              =  223, 
    kKey_Comm_Sat_4              =  224, 
    kKey_Comm_Sat_5              =  225, 
    kKey_Comm_Sat_6              =  226, 
    kKey_Comm_Sat_7              =  227, 
    kKey_Comm_Sat_8              =  228, 
    kKey_Comm_Sat_9              =  229, 
    kKey_Comm_Sat_10             =  230, 
    kKey_Comm_Sat_11             =  231, 
    kKey_Comm_Sat_12             =  232, 
    kKey_Program_Info            =  233, 
    kKey_Program_Nav             =  234, 
    kKey_View_Rec_Schedules      =  235, 
    kKey_Delete                  =  236, 
    kKey_Rotate_Right            =  237, 
    kKey_Rotate_Left             =  238, 
    kKey_Rotate_Center           =  239,
    kKey_Skype                   =  240,
    kKey_Time_Slip               =  241,
    kKey_Radio_TV                =  242,
    kKey_Counter_Program         =  243,
    kKey_Chapter_Mark            =  244
} QS_osm_key_t;

typedef enum {
    UNKNOWN = -1,
    WORKED = 0,
    DID_NOT_WORK = 1
} QS_osm_user_response_t;

typedef enum
{    
    kResult_GET_NEXT = 1, /* There are more keys to test, continue testing */
    kResult_SUCCESS = 2,  /* No more keys to test, successful */
    kResult_FAILED = 3,   /* Failed to get codeset from search, restart
                             whole process. */
    kResult_ERROR = -1
    
} QS_osm_test_result_response_t;

/* Optimized setup map row */
typedef struct
{
    char *codesetName;
    QS_u32 *patterns;
} OSMMapRow_t;

/* OSM Map */
typedef struct
{
    /* Optimized setup map */
    unsigned char mapVersion;
    int mapColumnCount;
    int mapRowCount;
    char **mapColumnNames;
    OSMMapRow_t *mapRowData;
    int primaryCodeset;
} OSMMap_t;

#ifndef _QS_PRELUDE
    #include <stdlib.h>
    #include <string.h>
    #include <stdio.h>
    #define TRUE 1
    #define FALSE 0
#else
    #include _QS_PRELUDE
#endif

#ifdef _GNU_EXT
    #include <stdlib.h>
    #include <string.h>
    #include <stdio.h>
#endif

#ifndef QS_LITTLE_ENDIAN_BYTE_ORDER
    #define QS_LITTLE_ENDIAN_BYTE_ORDER 1234
#endif
#ifndef QS_BIG_ENDIAN_BYTE_ORDER
    #define QS_BIG_ENDIAN_BYTE_ORDER 4321
#endif

#ifndef BYTE_ORDER
    #define BYTE_ORDER 1234
#endif

#if (BYTE_ORDER != QS_LITTLE_ENDIAN_BYTE_ORDER) && (BYTE_ORDER != QS_BIG_ENDIAN_BYTE_ORDER)
    #error The host system is an unsupported byte ordering!
#endif

/**
 * @brief                   This type indicates the method of 
 *                          logging debug messages. It should be
 *                          used when accessing logging APIs.
 *
 * QSLOGTYPE_NONE           Used when there is no output defined
 *
 * QSLOGTYPE_STDOUT         Send the output to stdout.
 *
 * QSLOGTYPE_FILE           Send the output to a file.
 *
 * QSLOGTYPE_BUFFER         Store the output in a buffer.
 *
 * QSLOGTYPE_ANDROID        Send the output to Android. 
 *  
 * QSLOGTYPE_WINDOWS        Send the output to Windows output.
 *
 * QSLOGTYPE_TIZEN          Send the output to Tizen.
 */
typedef enum {
    QSLOGTYPE_NONE,
    QSLOGTYPE_STDOUT,
    QSLOGTYPE_FILE,
    QSLOGTYPE_BUFFER,
    QSLOGTYPE_ANDROID,
    QSLOGTYPE_WINDOWS,
    QSLOGTYPE_TIZEN
} QS_log_type_t;

/**
 * @brief                           The region code type.
 *
 *    QS_region_code_type_zipcode   The U.S. zipcode type.
 * 
 */
typedef enum {
    QS_region_code_type_zipcode = 1
} QS_region_code_type_t;

/**
 * @brief                   This query mode data indicates the
 *                          current enabled local database query
 *                          methods.
 *
 *    QS_query_normal       This value indicates all supported query
 *                          methods are enabled.
 * 
 *    QS_query_no_model     This value indicates querying model
 *                          data is disabled.
 * 
 */
typedef enum {

    QS_query_invalid = -1,
    QS_query_normal = 0,
    QS_query_no_model

} QS_query_mode_t;


/**
 * @brief                   This flag instructs how to set
 *                          the blaster download threshold.
 *
 *   QS_query_reset_normal  Reset the counter, and set the
 *                          threshold to normal (100).
 * 
 *   QS_query_reset_test    Reset the counter, and set the
 *                          threshold to test level (5)..
 * 
 */
typedef enum {

    QS_query_reset_normal = 0,
    QS_query_reset_test

} QS_query_mode_reset_t;

/**
 * @brief        This flag defines how the data obtained from 
 *               the online database is to be treated.
 *
 *               QS_append      flag discards the data common to local database
 *                              and appends the rest to the internal data cache. 
 *               QS_replace     flag replaces the existing data list with the new
 *                              data from online database.
 *               QS_exclude_old flag discards the data common to local database
 *                              and replaces the internal cache with data only
 *                              available in the online database.
 */
typedef enum {

    QS_append = 0,
    QS_replace,
    QS_exclude_old

}QS_data_flag_t;

#ifdef _IR_LEARNER_
/*For IR learner*/
typedef struct QS_irb_waveform_s{
        int numToggleIndex;
        int dutyCycle;
        int carrierFrequency;

        int  numCharTimes;
        int *characteristicTime;

        int  numMakeTimes;
        int *makeTimes;

        int  numRepeatTimes;
        int *repeatTimes;

        int  numBreakTimes;
        int *breakTimes;
}QS_irb_waveform_t;
#endif

typedef ueic_status_t QSP_result_t;

typedef enum {
    QSP_session_mode_invalid,  /* Invalid state. */
    QSP_session_mode_local,    /* Local DB transactions only. */
    QSP_session_mode_online,   /* Online DB transactions only. */
    QSP_session_mode_auto,      /* Temp, will remove. */
} QSP_session_mode_t;

typedef QS_u32 QSP_session_t;

#ifndef LIBQS_API
#if __GNUC__ >= 4
#define LIBQS_API   __attribute__ ((visibility ("default")))
#elif defined(_WINDOWS) || defined(_WIN32)
#define LIBQS_API   __declspec(dllexport)
#else
#define LIBQS_API
#endif
#else
#define LIBQS_API
#endif

#ifndef LIBQSINT_API
#define LIBQSINT_API LIBQS_API
#endif

#endif /* _UEIC_DEFS_H */
