/**
 * @file        ueic_status.h
 * @brief       This file contains functions structures for
 *              reporting various types of run-time status or
 *              errors
 */

/* 
 * Copyright 2010 Universal Electronics Inc. All Rights
 * Reserved.
 * 
 * NOTICE:  You are permitted to use this file only in
 * accordance with the terms of the Universal Electronic Inc.
 * license agreement under which it was obtained.
 * Redistribution, publication, or distribution in any form is
 * not permitted without a separate license.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE COPYRIGHT OWNER(S) BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 */

#ifndef _ueic_status_H
#define _ueic_status_H

/**
 * @brief       Defines errors encountered in Quickset Embedded
 *              operations
 */
typedef enum ueic_status_e
{
    /** The operation was a success */
    ueic_success            = 0,

    /** The operation failed to initialize some subsystems**/
    ueic_failed_to_init     = 0x1,
    
    /** The operation encountered an uninitialized state when expecting and/or 
        requiring some initialized state */
    ueic_uninitialized_error= 0x2,

    /** The operation encountered an error and was uble to determine
     *  the cause of the error */
    ueic_unknown_error      = 0x3,

    /** The operation encountered an error that was not
     *  classified by this enum */
    ueic_general_error      = 0x4,

    /** The operation encountered some kind of network error */
    ueic_network_error      = 0x5,
    ueic_no_network_connection  = 0x6,
    ueic_unsupported_protocol   = 0x7,
    ueic_url_malformat          = 0x8,
    ueic_couldnt_resolve_host   = 0x9,
    ueic_couldnt_resolve_proxy  = 0x0A, /*10*/
    ueic_not_connected          = 0x0B,
    ueic_couldnt_connect        = 0x0C,
    ueic_remote_access_denied   = 0x0D,
    ueic_network_operation_timeout= 0x0E,
    ueic_network_send_err         = 0x0F,
    ueic_network_recv_err         = 0x10,  
    ueic_http_returned_err      = 0x11,
    ueic_http_post_error        = 0x12,



   /** The operation was aborted */
    ueic_operation_aborted      = 0x13,

    /** The operation encountered some kind of error while
     *  parsing an XML document */
    ueic_parser_error           = 0x14,/*20*/

    /** The operation encountered some kind of error when
     *  querying the internet database */
    ueic_onlinedb_query_error   = 0x16,

    /** The operation encountered some kind of unknown error
     *  querying the internet database */
    ueic_unknwn_onlinedb_error  = 0x17,

    /**  Unable to authenticate */
    ueic_auth_onlinedb_error    = 0x18,

    /** The operation encountered some kind of error when
     *  querying the database */
    ueic_database_error         = 0x19,

    /** The operation encountered an XMP2 error with the host
     *  sending the transceiver an invalid command*/
    ueic_xmp2_invalid_command   = 0x1A,

    /** The operation encountered an XMP2 error with the host
     *  sending the transceiver a bytestring with invalid
     *  checksum */
    ueic_xmp2_checksum_error    = 0x1B,

    /** The operation encountered an XMP2 error trying to read an
     *  illegal address in the remote FDRA */
    ueic_xmp2_invalid_address   = 0x1C,

    /** The operation encountered an XMP2 error with illegal
     *  parameters used in remote MCU command */
    ueic_xmp2_invalid_parameters= 0x1D,

    /** The operation encountered an XMP2 error where remote FDRA
     *  memory is full */
    ueic_xmp2_memory_full       = 0x1E, /*30*/

    /** The operation encountered an XMP2 error trying to program
     *  an upgrade image for the wrong device type into the wrong
     *  mode slot */
    ueic_xmp2_invalid_slot      = 0x1F,

    ueic_xmp2_invalid_device_type= 0x20,    /*32*/

    ueic_xmp2_invalid_device_code= 0x21,

    ueic_xmp2_invalid_key_code      = 0x22,

    /** The operation encountered an XMP2 error unrecognized
     *  command sent to the remote MCU */
    ueic_xmp2_invalid_host_command  = 0x25, 

    /** the operation encountered an XMP2 error with an invalid
     * checksum send from the host
     *  */
    ueic_xmp2_invalid_host_checksum = 0x26,

    /** The operation encountered an XMP2 error with no response
     *  from a remote */
    ueic_xmp2_no_remote_ack         = 0x27,

    /** the operation encountered a fatal error stemming from
     *  either internal checksum failure or byte size mismatch */
    ueic_xmp2_fatal_error           = 0x28,

    /** The operation encountered an XMP2 RS232 communication
     *  error most commonly permissions */
    ueic_xmp2_rs232_error           = 0x29,

    /** The operation encountered an XMP2 no data received
     *  from remote error */
    ueic_xmp2_no_remote_data        = 0x2A,

    /** The operation encountered an unknown XMP2 error */
    ueic_xmp2_general_error         = 0x2B,

    /** The operation could not find the requested data
     */
    ueic_data_not_found             = 0x2C,

    /** The operation encountered a pointer set to NULL which
     *  should have been set to a memory address */
    ueic_null_pointer_error         = 0x2D,

    /** The operation was not able to allocate memory */
    ueic_out_of_memory_error        = 0x2E,

    /** The operation encountered an I/O error */
    ueic_io_error                   = 0x2F,

    /** The operation could not find a required file in the file
     *  system */
    ueic_file_not_found             = 0x30,

    /** The the file is not in valid format   */
    ueic_file_format_error          = 0x31,

    /** The operation incountered an index not within a required
     *  range */
    ueic_index_out_of_bounds_error  = 0x32,

    /**
     * The operation incountered an invalid argument in a funtion
     * call
     */
    ueic_invalid_argument_error     = 0x33,

    /** The operation received an unexpected of invalid responce */
    ueic_invalid_response_error     = 0x34,

    /** The operation encounted the stub of an unimplemented
     *  function */
    ueic_unimplemented_error        = 0x35,

    /** The current functionality has been disabled */
    ueic_disabled_error             = 0x36,

    /** The operation tried to perform an action that it did not have permission 
     to perform */
     ueic_insufficient_permissions_error= 0x37,

   /** Configuration file error */
    ueic_config_file_error          = 0x38,

   /** Key not found within config file */
    ueic_config_key_not_found       = 0x39,

    /** The operation was performed while the system was in an invalid state */
    ueic_invalid_state_error        = 0x3A,
    
    /** The operation encountered multiple elements which should be mutual 
     *  exclusive */
    ueic_mutual_exclusivity_error    = 0x3B,

    ueic_ssl_connect_err        = 0x3C,
    ueic_ssl_certprblm_err      = 0x3D,
    ueic_ssl_cipher_err         = 0x3E,
    ueic_ssl_cacert_err         = 0x3F,
    ueic_ssl_cacert_badfile_err = 0x40,
    ueic_ssl_err                = 0x41,

    
    /* This error will happen when EDID Operations are performed and EDID is disable */
    ueic_auto_lookup_disable_err       = 0x42,

    ueic_already_exists_err     = 0x43,

    ueic_already_full_err       = 0x44,

    ueic_hdmi_device_not_supported   = 0x45,
    ueic_zlib_inflate_init_failed = 0x46,

    /**  Database version number is not supported */
    ueic_db_version_error    = 0x47,


    /** The size of the codeset exceeds the maximum allowed size,
     *  typically 800 bytes.
     */
    ueic_codeset_data_size_too_large= 0xC0,

    /** UEIX reported invalid data error. This error could be due to
     *  incorrect length or incorrect packet info.
     **/
    ueic_ueix_data_info_err         = 0xC1, 

    /** UEIX reported unsupported device type error.
     *  If the device number in the packet to be stored is greater
     *  than that supported by the internal DB, StoreCodeUpgrade
     *  returns this error code.
     * 
     **/
    ueic_ueix_device_not_supported  = 0xC2,

    /** UEIX reported blaster memory full error. Need to delete some not
     *  used codesets to free up memory space.
     *  */
    ueic_ueix_memory_full           = 0xC3,

    /** UEIX reported low voltage error. This is low voltage error
     *  while storing the info using StoreCodeUpgrade function.
     **/
    ueic_ueix_low_voltage           = 0xC4,

    /** UEIX reported ram exception error. Occurs when an API with
     *  invalid pointer is called.
     **/
    ueic_ueix_ram_exception            = 0xC5,

    /** UEIX reported an invalid device code */
    ueic_ueix_invalid_device_code    = 0xC6,

    /** UEIX reported an invalid device type */
    ueic_ueix_invalid_device_type    = 0xC7,

    /** UEIX reported invalid key code */
    ueic_ueix_invalid_key_code        = 0xC8,

    /** UEIX reported bad fdra */
    ueic_ueix_bad_fdra            = 0xC9,

    /** UEIX reported learning error */
    ueic_ueix_learning_error        = 0xCA,

    /** UEIX reported downloaded code already exists */
    ueic_ueix_downloaded_already_exists = 0xCB,

    /** UEIX reported invalid learn code ID used */
    ueic_ueix_invalid_learncodeID    = 0xCC,

    /** Online Request is Aborted by Callback due to User Request */
    ueic_online_aborted_by_callback = 0xCD,

    /** Invalid tag found in config xml. */
    ueic_unable_to_load_xml = 0xCE

} ueic_status_t;

/**
 * @brief       Translates an error code into a human-readable
 *              string
 * @param err   the error code to translate
 * 
 * @return      const char* - the string representation of the
 *              error code
 */
const char* ueic_status_errorToString(ueic_status_t err);







#endif
